/*3456789_123456789_123456789_123456789_123456789_123456789_123456789_12345678*/
/***MAGEMAIN.C***************************************************************/
/*MAGE
 * shows KINEMAGEs
 *
 *  MAGE: Attribution Notice:
 * MAGE program: copyright c David C. Richardson, 1991,1992,1993,1994
 *                                     all rights reserved
 *            Little River Institute, 5820 Old Stony Way, Durham, NC 27705
 *
 * First use was for the Interactive Graphics Appendix to Protein Science,
 *                                           Journal of the Protein Society.
 * Use and distribution for non-profit educational and research purposes
 * encouraged.  Rewrites for this and other platforms encouraged for similar
 * use and distribution.
 *            David C. Richardson, dcr@suna.biochem.duke.edu
 *    Biochemistry Dept., Duke University, Durham, North Carolina 27710, USA
 * Concept and MAGE   Developed by David C. Richardson and Jane S. Richardson
 *        "Kinemage" name invented by Caroline P. Usher
 * Grateful acknowledgement of many years of support from NIH, grant GM15000
 * which included visualization of protein structures and the development of
 * CHAOS: display program written in E&S PS300 function-net language,
 * CHAosIN Fortran program,  etc.
 *
 *   Although this code obviously bears resemblance to many examples of
 *    .c code, one hopes this combination and construction departs from the
 *    examples far enough so that there is no danger of copywrite infringement,
 *    nor blame for failings, going back to earlier work.
 *   On the other hand, there is a real intellectual debt owed.
 *   In particular:
 *     Think C development system, examples, and manual, Symantec Corp. 1989 +
 *            and several timely hints from Think C technical Support!
 *     Apple Inside Macintosh, Addison-Wesley, 1985
 *          (the essentual and eventually preferred ref)
 *     Several suggestions and a look at real code by Warren Robinett
 *     Using the Macintosh Toolbox with C,
 *            F.A.Huxham, D.Burnard,J.Takatsuka 2nd Ed. Sybex,1989
 *     Macintosh C Programming Primer, Vol. II, D. Mark, Addison-Wesley, 1990
 *     Macintosh C Programming Primer, Vol. I 2nd Ed,  Mark & Reed, 
 *            Addison-Wesley,1992.  Particularly for AppleEvents
 *     Roger Sayle, RasMol author, for making rasmol produce kinemage files,
 *         and whose code showed me really how to use Apple Events!
 *     Robert Weiss who began making useful suggestions even before he signed
 *         on to help rewrite MAGE for the PC. (== coauthor for PC_MAGE)
 *     Borland C++, Borland International, Inc. 1991
 *      (but MAGE tries to be vanilla C as much as possible)
 *     Microsoft Windows Programmer's Reference, for version 3,  1990
 *     UNIX and MOTIF ...(add refs here)...
 *
 *  Note: \015 == CR imbedded in text, e.g.: sprintf(s," line1 \015 line2 ")
 *   sometimes use \n for newline instead. CR LF worst compatibilities...
 * CRLF defined as "\015" (\r) for MAC, PC, "\012" (\n) for UNIX. 950110 
 * Files: various MAGE____.c routines, MAC____.c, MPC____.c, etc.
 *  &  platform specific libraries like: ANSI & MacTraps from THINK-C library.
 * Mac Files are grouped, esp Main,Draw that call each other during
 *  critical speed use, i.e. for the real time rotation.
 * Files are Refered to by comments. 
 * AMAC... is for mac routines that can't be copied over to PC robustly.
 * MAC...  is for mac routines that can be copied.
 * MAGE... is for routines used by both Mac and PC.
 * MPC...  is for PC routines that can be copied.
 * PCW...  is for PC Windows routines that probably can't be copied robustly.
 * UNIX versions are even more confusing, see top of MAGE.H
 *
 *  Coding follows Allman (BSD) indent style (Eric Allman, Berkeley)
 *  With some attempt to lead and trail subroutines so they can be seen easily
 *  and try to stay <=80 col so any editor, printer, or email is robust.
 *
 *Subroutine calls flagged by comment indicating source file of subroutine,
 * or if a C routine, etc. Beware: this is not always accurate! 
 * Some common C functions, like sprintf, are not always flagged.
 *To translate from English: alertstr[256],alertstr2[256],alertstr3[256] used 
 * for comments to the reader.  Sometimes word[256] printed to alertstr2[256].
 * AMACMAGE.proj.rsrc and PCWMAGE.RC need translation.
 * check other sprintf calls here in MAGEINIT.C  
 ****************************************************************************/
/*3456789_123456789_123456789_123456789_123456789_123456789_123456789_12345678*/
/*MAGEINIT.C*****************************************************************/
/*initial routines */

#include "MAGE.H"

/****getmageversion()*****************************************************/
void getmageversion()   /*common version number: MAGEMAIN.C */
{
         sprintf(temps,"dated: 950125");
         sprintf(word,"alpha version 4.0");
#ifdef EGAM
         sprintf(version,"EGAM version %s, %s",word,temps);
#else
         sprintf(version,"MAGE version %s, %s",word,temps);
#endif
#ifdef PCWINDOWS
    sprintf(modification,"  PC MAGE written by "
         "Robert M. Weiss and David C. Richardson");
#endif
#ifdef UNIX_X11
    sprintf(modification," UNIX GENERIC alpha test");
#endif
#ifdef UNIX_PEX
    sprintf(modification," UNIX PEX alpha test");
#endif
}
/*___getmageversion()____________________________________________________*/

/****mageinitialvalues()***************************************************/
void    mageinitialvalues()
{
    int    i,n;
  /*------------initial dialog allows change--------------------------*/
    MAXNUM = 3500;   /*atoms and points total stored per kinemage*/
    MAXDRAWS = 3500; /*moveto and drawto points at any time*/
  /*------------------------------------------------------------------*/
    MAXWORDCOM = 23500; /*total number of characters comment or wordlist*/
    nwordcom = 1; /*starts at 1, since 0 means no comment or wordlist*/
  /*--set array pointers==NULL so can tell if allocate them-----------*/
    for (i = 0; i < 3; ++i)fxyz[i] = NULL;
    for (i = 0; i < 5; ++i)xyz[i] = NULL;  /*up from 4*/
    for (i = 0; i < 8; ++i)it[i] = NULL;
    com = NULL;
    wordcom = NULL;
    inwordcom = NULL;
  /*------------------------------------------------------------------*/
    
    maxValue=10000;
    minValue = 0;
    width = 5;
if(IUX)
{
    GWIDTH = 800;   /* UNIX effective graphics area window width */
    GHEIGHT = 800;  /* graphics window height*/
    GZDEEP = 800;
}
else /*if(IMAC||IPC)*/
{
    GWIDTH = 400;   /* Mac effective graphics area window width */
    GHEIGHT = 400;  /* graphics window height*/
    GZDEEP = 400;
}
    WIDTH =  GWIDTH; /*GWIDTH & GHEIGHT adjusted when make grafWindow*/
    HEIGHT = GHEIGHT;/* WIDTH & HEIGHT reset then also */

    ISOSCALE = GWIDTH/2; 
      /*PC graphics window scale for initial width, height*/
      /*PC does everything in terms of ISOSCALE so fzclip==1.0 always */
    maxwide=GWIDTH;
    CHEIGHT = 80;   /* caption window height */
    MHEIGHT = 60;   /* message place height (UNIX) */
if(IUX)
{
    BWIDTH = 200;   /* buttons,slider area width in graphics window*/
    ZWIDTH =  60;   /* 30 for zoom, zclip, ztran scroll bars */
}
else
{
    BWIDTH = 100;   /* buttons,slider area width in Mac graphics window*/
                    /*on Mac graphics screen: 100 for buttons with labels */
    ZWIDTH =  30;   /* 30 for zoom, zclip, ztran scroll bars */
}
    RWIDTH =   0;   /*variable:  0 initially for bondrotation scroll bars */
    ROTSWIDTH=100;  /*Mac allowance for bondrotation scroll bars, PC resets*/
    ROTWINDWIDTH=200; /*constant for bondrotsliders separate UNIX window*/
    izclip = 200;
        /*std screen box 400x400x400, 0 at ctr, zclip 200 from ctr*/
    /*fzclip = 1.0;*/ /*factor==1.0 for std box, scale if screen box resized */
    fzclip = (float)GZDEEP/(float)400;/*std izclip defined in std screen box*/
    /*fzclip mediates between internal izclip value adjusted for box expansion*/
    /* and external izclip which pretends to be in a 400x400x400 box */
    zoom = 1.0;
    iztran = 0;

    defaultangle = 6.0;
    stereoangle = defaultangle;
    ieyedefault = GWIDTH/2;
    ieyewide = ieyedefault;
    Lmiddle = 1;
    Lrecalculate = 0;
    Lmultibins = 0;
    multibins = 1;
    Lmoview = 0;
    Lkeepstereo = 0;
    Lkeepthinline = 0;
    scale = 1.0;

    nbondrot = 0;  /*initially should be 0, no provision for bond rotation*/
    distuser = 0.0;  /*distance and angle preferences for constructing lines*/
    angleuser = 0.0;
    dihedraluser = 0.0;
    diststored = 1.54; /*distance and angles stored for construction*/
    anglestored = 109.6;
    dihedralstored = 60.0;
    Ldrawunpickable = 1;
    shortenline = 0.0; /*draw new amount to shorten lines, e.g. H-bonds*/
    /*PC needed variables
    if(IPC) /*defined differently for PC */
    {
        Loriginatctr = 1;
        ixctr = 0;
        iyctr = 0;
        izctr = 0;
    }
    ibigfont = 0;  /*ibigtype=40 for PC with clunky old big type font*/
           /* moves the rightside screen messages ibigfont pixels left*/

    zoomfac = 1.0; /*slider uses this directly*/
    slabfac = 2.0; /*slider uses this * 100*/
    tranfac =0.0;  /*slider uses this * 100*/
    Initialized = 0;
    SliderWidth = 0;

    /*Macintosh needed variables */
    if(IMAC||IUX) /*defined differently for Mac */ /*UNIX*/
    {
        Loriginatctr = 0;
        ixctr = GWIDTH/2;
        iyctr = GHEIGHT/2;
        izctr = GZDEEP/2;
    }
    textHeight = 1;
    captHeight = 1;
    widMax = 1;
    nxyz = 1;
    nmovedraws=0;
    ntotaltext=0;
    
    sprintf(draglinestr,"dragline");
    condition(draglinestr);
    
    Lpostscript = 0;
    Lspecialloop = 0;
    Lnewlabels = 0;
    Lmonitor = 0;
    Lprocessing = 0;
    Liveoffscreenbitmap = 0; /*flag for off-screen bitmap */ 
    Lquesanswers = 0; /*flag for questions in text part of kinemage*/
    for(n=0; n<256; n++) alertstr[n] = '\0';  /* clear alert report */
    for(n=0; n<256; n++) word[n] = '\0';  /* clear alert report */
    for(n=0; n<256; n++) temps[n] = '\0';  /* clear alert report */ 

#ifdef PCWINDOWS  
    hasnewDC = 0;
    ghtextbuf = 0;
    textsize = 0;
    textuse = 0;
    ghcaptsbuf = 0;
    captssize = 0;
    captsuse = 0;
#endif
}
/*___mageinitialvalues()____________________________________________________*/
/*3456789_123456789_123456789_123456789_123456789_123456789_123456789_12345678*/
/****setMageinitDLOGreport()*************************************************/
void    setMageinitDLOGreport(void)
{                                          /*950110 CRLF */
            sprintf(alertstr,
            "MAGE: copyright \251 1995 by David C. Richardson, "
            CRLF"    Little River Institute, 5820 Old Stony Way, "
                   "Durham NC 27705 "
            CRLF"       dcr@suna.biochem.duke.edu"      
            CRLF"Biochemistry Dept., Duke University, "
                   "North Carolina 27710, USA"
            );
            sprintf(alertstr2,
            CRLF"%s"
            CRLF"%s"
            CRLF CRLF"MAGE may be freely copied and distributed."
            ,version,modification
            );
            sprintf(alertstr3," "); 
}            
/*___setMageinitDLOGreport()________________________________________________*/
/*3456789_123456789_123456789_123456789_123456789_123456789_123456789_12345678*/
/****clearfocusxyzstr()******************************************************/
void    clearfocusxyzstr()
{
    sprintf(focusxyzstr,"\0"); /* clear focusxyzstr for it to be used */
    condition(focusxyzstr);       /* for x,y,z report on graph screen */
}
/*___clearfocusxyzstr()_____________________________________________________*/
/*3456789_123456789_123456789_123456789_123456789_123456789_123456789_12345678*/

/*3456789_123456789_123456789_123456789_123456789_123456789_123456789_12345678*/
/****allocdataarrays()*******************************************************/
void    allocdataarrays(int more)
{
/*these arrays come in sets that have to be of the same size since they*/
/* are indexed the same way, and subject to random access */
/*MAXNUM limits all arrays of MAGE's 3-D data organization*/
/*These are: fxyz[3],xyz[5],com,inwordcom */
/*MAXDRAWS limits all arrays of GENERIC MAGE's 2-D display list*/
/*These are: it[8] */
/*wordcom is independent, though indexed into by entries in inwordcom */

    int  j, ifail, recycle;
    char *comtemp;
    int *xyztemp[5];
    float *fxyztemp[3];
    int *inwordcomtemp;
    long maxcom;

    /*allocate pseudo array, THINK C user's manual, 1989, pg 450-451 */
    /*but using malloc instead of NewPtr so will work for all platforms*/
    /* substitutes for float fxyz[MAXNUM][3];, note changing index order */
    /* uses this as separate way to allocate memory in Heap beyond */
    /* programs 32k data or each segment's 8k of stack space */
    /*more==0 for first call to allocate initial arrays*/

    recycle = 1;
    while(recycle==1)
    {/*recycle allocation trials*/
        ifail = 0;
        for(j = 0; j < 3 ; j++)
        {  
            if(more==0)
            {
                fxyz[j] = (float *)malloc(sizeof(float)*MAXNUM);
                if(fxyz[j]==NULL) ifail = 1;
            }
            else
            {
              fxyztemp[j]=(float *)realloc(fxyz[j],sizeof(float)*(MAXNUM+more));
              if(fxyztemp[j]==NULL) ifail = 1;
            }
        }
        for(j = 0; j < 5 ; j++) 
        { 
            if(more==0)
            {
                xyz[j] = (int *)malloc(sizeof(int)*MAXNUM);
                if(xyz[j]==NULL) ifail = 1;
            }
            else
            {
                xyztemp[j] = (int *)realloc(xyz[j],sizeof(int)*(MAXNUM+more));
                if(xyztemp[j]==NULL) ifail = 1;
            }
        }
        if(more==0)
        {
            maxcom = (long)MAXNUM*(long)(MAXPTIDCHAR+1);
            com = (char *)malloc(sizeof(char)*maxcom) ;
            if(com==NULL) ifail = 1;
            inwordcom = (int *)malloc(sizeof(int)*MAXNUM);
            if(inwordcom==NULL) ifail = 1;
        }
        else
        {
            maxcom = (long)(MAXNUM+more)*(long)(MAXPTIDCHAR+1);
            comtemp = (char *)realloc(com,sizeof(char)*maxcom) ;
            if(comtemp==NULL) ifail = 1;
            inwordcomtemp = (int *)realloc(inwordcom,sizeof(int)*(MAXNUM+more));
            if(inwordcomtemp==NULL) ifail = 1;
        }
        if(more>0 && ifail==1)
        {/*failed to do a reallocation, reduce request and try again*/
            more = more/2;
            if(more > 3) recycle = 1;
            else recycle = 0;
        }
        else recycle = 0;
    }/*recycle allocation trials*/

    if(more>0 && ifail==0)
    {/*reassign pointers*/
        for(j=0;j<=2;j++) fxyz[j] = fxyztemp[j];
        for(j=0;j<=4;j++)  xyz[j] =  xyztemp[j];
        com = comtemp;
        inwordcom = inwordcomtemp;
        /*increase limit*/
        MAXNUM = MAXNUM+more;
        /*printf("increased MAXNUM to %d\n",MAXNUM);*/ /*950113*/
    }
    if(Ltest||ifail)
    {
        if(more>0 && ifail>0)
        sprintf(alertstr,"reallocation failed, remains:\n"
         "alloc: fxyz[0]== %ld, fxyz[1]== %ld, fxyz[2]== %ld"
         ,fxyz[0],fxyz[1],fxyz[2] ); 
        else
        sprintf(alertstr,"alloc: fxyz[0]== %ld, fxyz[1]== %ld, fxyz[2]== %ld"
         ,fxyz[0],fxyz[1],fxyz[2] ); 
        sprintf(alertstr2,"alloc: xyz[0]==%ld,xyz[1]==%ld,xyz[2]==%ld"
         ",xyz[3]==%ld,xyz[4]==%ld"
         ,xyz[0],xyz[1],xyz[2],xyz[3],xyz[4]); 
        sprintf(alertstr3,"alloc: com==%ld, inwordcom==%ld"
         ,com,inwordcom ); 
        DoMageDLOGreport(); /*___DLOG.C*/ /*alertstr,alertstr2,alertstr3*/
    }
    if(ifail&&more==0)
    {
        sprintf(alertstr,"allocation of more data space failed");  /*950110*/
        sprintf(alertstr2,"expect truncated image");
#ifdef MACINTOSH
        sprintf(alertstr3,"Perhaps preferred size too small: Before launching"
                          " MAGE:"
                          CRLF"one click on MAGE, choose Info under File Menu: "
                          CRLF"change preferred size to, e.g., 2000 "
                          );
#else
        sprintf(alertstr3," ");
#endif
        DoMageDLOGreport(); /*___DLOG.C*/ /*alertstr,alertstr2,alertstr3*/
        /*cleanup();*/
        /*exit(EXIT_FAILURE);*/
    }
}
/*___allocdataarrays()_______________________________________________________*/

/****allocdisplayarrays()****************************************************/
void    allocdisplayarrays(int more)
{

/*    int    it[8][MAXDRAWS];  */   /* size = 2*MAXDRAWS*8 9310:7->8 */

    /*allocate pseudo array, see comments in allocdataarays() */

    int  j, ifail, recycle;
    int *ittemp[8];

    /*more==0 for first call to allocate initial arrays*/

    recycle = 1;
    while(recycle==1)
    {/*recycle allocation trials*/
        ifail = 0;
        for(j = 0; j < 8 ; j++)
        {  
            if(more==0)
            {
                it[j] = (int *)malloc(sizeof(int)*MAXDRAWS);
                if(it[j]==NULL) ifail = 1;
            }
            else
            {
                ittemp[j] = (int *)realloc(it[j],sizeof(int)*(MAXDRAWS+more));
                if(ittemp[j]==NULL) ifail = 1;
            }
        }
        if(more>0 && ifail==1)
        {/*failed to do a reallocation, reduce request and try again*/
            more = more/2;
            if(more > 3) recycle = 1;
            else recycle = 0;
        }
        else recycle = 0;
    }/*recycle allocation trials*/

    if(more>0 && ifail==0)
    {/*reassign pointers*/
        for(j=0;j< 8;j++) it[j] = ittemp[j];
        /*increase limit*/
        MAXDRAWS = MAXDRAWS+more;
        /*printf("increased MAXDRAWS to %d\n",MAXDRAWS);*/ /*950113*/
    }
    if(Ltest||ifail)
    {
        if(more>0 && ifail>0)
            sprintf(alertstr,"reallocation failed, remains:\n"); 
        else
            sprintf(alertstr," "); 
        sprintf(alertstr2,"alloc: it[0]==%ld, it[1]==%ld, it[2]==%ld, it[3]==%ld\n"
           "alloc: it[4]==%ld, it[5]==%ld, it[6]==%ld, it[7]==%ld\n"
         ,it[0],it[1],it[2],it[3],it[4],it[5],it[6],it[7] ); 
        sprintf(alertstr3," "); 
        DoMageDLOGreport(); /*___DLOG.C*/ /*alertstr,alertstr2,alertstr3*/
    }
    if(ifail&&more==0)
    {
        sprintf(alertstr,"allocation of more display space failed");  /*950110*/
        sprintf(alertstr2,"expect truncated image");
#ifdef MACINTOSH
        sprintf(alertstr3,"Perhaps preferred size too small: Before launching MAGE:"
                          CRLF"one click on MAGE, choose Info under File Menu: "
                          CRLF"change preferred size to, e.g., 2000 ");
#else
        sprintf(alertstr3," ");
#endif
        DoMageDLOGreport(); /*___DLOG.C*/ /*alertstr,alertstr2,alertstr3*/
        /*cleanup();*/
        /*exit(EXIT_FAILURE);*/
    }
}
/*___allocdisplayarrays()___________________________________________________*/

/****allocwordcomarray()*****************************************************/
void    allocwordcomarray(int more)
{
/* char  *wordcom;  pointer to array, for wordcom[MAXWORDCOM], size=MAXWORDCOM*/
/* the comment or wordlist character array  */
    /*allocate pseudo array, see comments in allocdataarays() */

    long    maxcom;
    int  j, ifail, recycle;
    char *wordcomtemp;

    /*more==0 for first call to allocate initial arrays*/

    recycle = 1;
    while(recycle==1)
    {/*recycle allocation trials*/
        ifail = 0;
            if(more==0)
            {
                wordcom = (char *)malloc(sizeof(char)*MAXWORDCOM);
                if(wordcom==NULL) ifail = 1;
            }
            else
            {
            wordcomtemp=(char *)realloc(wordcom,sizeof(char)*(MAXWORDCOM+more));
            if(wordcomtemp==NULL) ifail = 1;
            }
        if(more>0 && ifail==1)
        {/*failed to do a reallocation, reduce request and try again*/
            more = more/2;
            if(more > 3) recycle = 1;
            else recycle = 0;
        }
        else recycle = 0;
    }/*recycle allocation trials*/

    if(more>0 && ifail==0)
    {/*reassign pointer*/
        wordcom = wordcomtemp;
        /*increase limit*/
        MAXWORDCOM = MAXWORDCOM + more;
    }
    if(Ltest||ifail)
    {
        if(more>0 && ifail>0)
            sprintf(alertstr,"reallocation failed, remains:");
        else
            sprintf(alertstr," "); 
        sprintf(alertstr2,"alloc: wordcom==%ld",wordcom ); 
        sprintf(alertstr3," "); 
        DoMageDLOGreport(); /*___DLOG.C*/  /*alertstr,alertstr2,alertstr3*/
    }
    if(ifail&&more==0)
    {
        sprintf(alertstr,"allocation of more comment space failed");  /*950110*/
        sprintf(alertstr2,"expect truncated words");
#ifdef MACINTOSH
        sprintf(alertstr3,"Perhaps preferred size too small: Before launching MAGE:"
                          CRLF"one click on MAGE, choose Info under File Menu: "
                          CRLF"change preferred size to, e.g., 2000 ");
#else
        sprintf(alertstr3," ");
#endif
        DoMageDLOGreport(); /*___DLOG.C*/ /*alertstr,alertstr2,alertstr3*/
        /*cleanup();*/
        /*exit(EXIT_FAILURE);*/
    }
}
/*___allocwordcomarray()____________________________________________________*/

/*3456789_123456789_123456789_123456789_123456789_123456789_123456789_12345678*/
/****cleanup()***************************************************************/
void cleanup(void)  
{
    int i,j,nt;

    for (i = 0; i < 3; ++i)
    {
        if (fxyz[i] != NULL) free(fxyz[i]);
    }
    for (i = 0; i < 5; ++i)
    {
        if (xyz[i] != NULL) free(xyz[i]);
    }
    for (i = 0; i < 8; ++i)
    {
        if (it[i] != NULL) free(it[i]);
    }
    if (com != NULL) free(com);
    if (wordcom != NULL) free(wordcom);
    if (inwordcom != NULL) free(inwordcom);

#ifdef PCWINDOWS
    if (hFileContents != 0) GlobalFree(hFileContents);
    if (ghcaptsbuf != 0) GlobalFree(ghcaptsbuf);
    if (ghtextbuf != 0) GlobalFree(ghtextbuf);
    if (lpPalette != NULL) GlobalUnlock(hPalette);
    if (hPalette != NULL) GlobalFree(hPalette);
#endif /*PCWINDOWS*/
}
/*___cleanup()______________________________________________________________*/
/*3456789_123456789_123456789_123456789_123456789_123456789_123456789_12345678*/
