/*MUXMINIT.c*/  /* MOTIF widgets */
/*3456789_123456789_123456789_123456789_123456789_123456789_123456789_12345678*/
/*
  mainsetup
  The mainsetup routine creates the widget hierarchy for the program.
  The widget hierarchy used here is:

  (class topLevelShell)
     |
     +-- form (class Form)
           |
	   +--- drawArea (class DrawingArea)
	   |

  The Form widget is used for organizing its child widgets.
  The drawing area widget is used for X11 draws or PEX operations and
  the PEX workstation is opened on its window.  The MAGE toggle buttons are
  used to control the parts of the image displayed.  The menu items control
  other functions like file select and quit operations.

  Any necessary resources for the widgets are specified here in the
  program, which override any user defaults or command-line options.  Note
  that this is not very friendly to the user who may want to change the
  font of the push buttons, but MAGE traditionally has had to control its
  own appearance.  

  The display connection (dpy) and window ID (drawWindow) of the drawing
  area widget are available after the widget hierarchy has been realized.
*******************************************************************************
  BEWARE!! BEWARE!!
    The drawing area widget in Motif 1.0 has a bug in that it ignores
    height and width resources supplied at creation time.  A workaround
    I've found is to set the margins of the drawing area to be half the
    desired height and width.  Since the margins specify the boundary
    between the drawing area widget's border and any children of the
    drawing area widget (we have none here), the drawing area widget will
    be sized to contain its children plus twice the margins in each
    direction.  Hence to get a 600x600 drawing area widget, you can set the
    margins to 300.  Other workarounds suggested involve creating the
    drawing area widget as a child of other widgets, but where there are no
    children of the drawing area, I prefer setting the margins.
       		Rich Thomson
                Design Systems Division
		Evans & Sutherland Computer Corporation
*******************************************************************************
*/
 /* 
      Once upon a time this file started with code bundled with the
  ESV, though it has been hacked much beyond recognition, some of the
  motif calls and setups here owe recognition to:
       		Rich Thomson
                Design Systems Division
		Evans & Sutherland Computer Corporation
         Date:  Thursday, June 12th, 1990
       Modified:  J. Buckmiller   Mar 1991.   Approved C binding
  Dave Richardson, June 1994
*/
  
#include "MAGE.H"       

#ifdef UNIX_PEX
 /* these event handlers take care of blanking and restoring the dial labels */
 extern void enter_handler(Widget, caddr_t, XEvent *,Boolean *); /*MUXPHIG.c*/
 extern void leave_handler(Widget, caddr_t, XEvent *,Boolean *); /*MUXPHIG.c*/
 /*defined for drawArea*/
#endif /*UNIX_PEX*/

/****mainsetup()*************************************************************/
void    mainsetup(int argc, char** argv)
{
  int     copy_of_argc, argv_size;
  char**  copy_of_argv;

  XmFontList fontList;
  int    i,j,k,ncommon;
  Dimension width, height;

/* #define SHORTWAY */
#define LONGWAY
#define COLORMESS

  getmageversion(); /*common version number: MAGEINIT.c */

  definecursors();  /*MUXMINIT.C*/
  makecursorwait(); /*MUX_MAIN.C*/

    IMAC = 0; /*flow control for Macintosh computers */
    IPC  = 0;  /*flow control for PC computers with windows */
    IESV = 0;  /*flow control for E&S ESV Unix with  Xwindows */
    IUXM = 1;  /*flow control for  Unix with Xwindows and Motif*/
    IUX  = 1;  /*flow control for  Unix with Xwindows*/
    IUXP = 0;  /*flow control for  Unix with Xwindows and PEX*/
  mageinitialvalues(); /*mageinit.c*/ /*in case need MAGE parameters*/



#ifdef LONGWAY
  /*Initialize X toolkit the long way so can mess with color palette */
  XtToolkitInitialize();
  app_context = XtCreateApplicationContext();
  /*copy commandline parameters*/
  copy_of_argc = argc;
  argv_size = argc * (sizeof(char*));
  copy_of_argv = (char**)XtMalloc(argv_size);
  memcpy(copy_of_argv, argv, argv_size);

  /*open display connection*/
  dpy = XtOpenDisplay(app_context,
        NULL,
        NULL,
        "MAGE",
        (XrmOptionDescList) NULL,
        0,
        &argc,
        argv);

  /*Now can set visual stuff before opening top shell */

#ifdef COLORMESS
/*discover Visual attributes of this computer */
#ifdef UNIX_PEX
  magevisual = DefaultVisual(dpy,DefaultScreen(dpy));
  magedepth = DefaultDepth(dpy,DefaultScreen(dpy));
  colormap = DefaultColormap(dpy,DefaultScreen(dpy));
  windowbackground = WhitePixel(dpy,DefaultScreen(dpy));
  windowborder = BlackPixel(dpy,DefaultScreen(dpy));
#else
  findmonitortype(); /*calls makexcolors()*/ /*MUX_COLR.c*/
#endif

#endif /*COLORMESS*/

  /* create Application Shell for mage named topLevel */
  /* set in commandline parameters */
  n = 0;
#ifdef COLORMESS
  XtSetArg(args[n], XmNvisual, magevisual); n++;
  XtSetArg(args[n], XmNdepth, magedepth); n++;
  XtSetArg(args[n], XmNcolormap, colormap); n++;
  XtSetArg(args[n], XmNbackground, windowbackground); n++;
  XtSetArg(args[n], XmNborderColor, windowborder); n++;
#endif /*COLORMESS*/
  XtSetArg(args[n], XmNargc, copy_of_argc); n++;
  XtSetArg(args[n], XmNargv, copy_of_argv); n++;
  XtSetArg(args[n], XmNallowResize, True); n++;
  XtSetArg(args[n], XmNmappedWhenManaged, False); n++;
  XtSetArg(args[n], XmNtitle, "MAGE"); n++;
  XtSetArg(args[n], XmNx, 100); n++;
  XtSetArg(args[n], XmNy, 20); n++;
  XtSetArg(args[n], XmNwidth, GWIDTH+BWIDTH+ZWIDTH); n++; 
                         /*orig==800, 100,   30,  UNIX not use RWIDTH here */
  XtSetArg(args[n], XmNheight,GHEIGHT+MHEIGHT); n++; /*orig==860,GHEIGHT==800*/
  /*topgraf = XtAppCreateShell(NULL,*/
  topLevel = XtAppCreateShell(NULL,
                             "MAGE",
                             applicationShellWidgetClass,
                             dpy,
                             args,n);
#endif /*LONGWAY*/

#ifdef SHORTWAY

  /* create topLevelShell */
/*
  topLevel = XtInitialize(ProgramName, "MAGE", NULL, 0, &argc, argv);
*/
  n = 0;
  topLevel = XtAppInitialize(&app_context,"MAGE", NULL, 0, 
          &argc, argv,NULL,args,n);
#endif /*SHORTWAY*/


  /*create topgraf as a popup shell under topLevel*/
  n = 0;
#ifdef COLORMESS
  XtSetArg(args[n], XmNvisual, magevisual); n++;
  XtSetArg(args[n], XmNdepth, magedepth); n++;
  XtSetArg(args[n], XmNcolormap, colormap); n++;
  XtSetArg(args[n], XmNbackground, windowbackground); n++;
  XtSetArg(args[n], XmNborderColor, windowborder); n++;
#endif /*COLORMESS*/
  XtSetArg(args[n], XmNtitle, "MAGE GRAPHICS"); n++;
  XtSetArg(args[n], XmNx, 100); n++;
  XtSetArg(args[n], XmNy, 20); n++;
  XtSetArg(args[n], XmNwidth, GWIDTH+BWIDTH+ZWIDTH); n++; 
  XtSetArg(args[n], XmNheight,GHEIGHT+MHEIGHT); n++; /*orig==860,GHEIGHT==800*/
  XtSetArg(args[n], XmNallowResize, True); n++;
  /*XtSetArg(args[n], XmNmappedWhenManaged, False); n++;*/
  topgraf = XtCreatePopupShell("graf",
                               topLevelShellWidgetClass,
                               topLevel,
                               args, n);
  /*XtSetValues(topgraf, args, n);*/

  /*create grafwindow as a form widget to hold drawArea, menus, buttons, etc.*/
  n = 0;  /* window title is a regular string */
#ifdef COLORMESS
  XtSetArg(args[n], XmNvisual, magevisual); n++;
  XtSetArg(args[n], XmNdepth, magedepth); n++;
  XtSetArg(args[n], XmNcolormap, colormap); n++;
  XtSetArg(args[n], XmNbackground, windowbackground); n++;
  XtSetArg(args[n], XmNborderColor, windowborder); n++;
#endif /*COLORMESS*/
  XtSetArg(args[n], XmNtitle, "GRAPHICS"); n++;
  XtSetArg(args[n], XmNx, 100); n++;
  XtSetArg(args[n], XmNy, 20); n++;
  XtSetArg(args[n], XmNwidth, GWIDTH+BWIDTH+ZWIDTH); n++;
  XtSetArg(args[n], XmNheight,GHEIGHT+MHEIGHT); n++; /*orig==860,GHEIGHT==800*/

  buttonFont =				/* find the font we want for buttons */
    XLoadQueryFont(XtDisplay(topgraf), "-*-Helvetica-Bold-R-Normal--14*");
  if (buttonFont)
    fontList = XmFontListCreate(buttonFont, XmSTRING_DEFAULT_CHARSET);

  grafwindow =				/* create the grafwindow for layout */
    XtCreateManagedWidget("graff",xmFormWidgetClass,topgraf, args, n);
    /*XmCreateForm(topgraf,"grafwindow", args, n);*/
    /*XtCreateWidget("grafwindow", xmFormWidgetClass, topgraf, NULL, 0);*/
  /*XtManageChild(grafwindow);*/
#ifdef UNIX_PEX
  XtAddEventHandler(grafwindow, Button1MotionMask, False, motion_handler, NULL);
  /*motion_handler and pick_CB in MUX_MAIN.c for consistency of ___MAIN stuff*/
  /*motion by dials even when move cursor out of drawArea to hit buttons*/
  /*these event handlers take care of blanking and restoring the dial labels */
  XtAddEventHandler(grafwindow, EnterWindowMask, False, enter_handler, NULL);
  XtAddEventHandler(grafwindow, LeaveWindowMask, False, leave_handler, NULL);
#endif

/***********************MENU************************************************/
/*create menuBar*/
  n = 0;
#ifdef COLORMESS
  XtSetArg(args[n], XmNvisual, magevisual); n++;
  XtSetArg(args[n], XmNdepth, magedepth); n++;
  XtSetArg(args[n], XmNcolormap, colormap); n++;
  XtSetArg(args[n], XmNbackground, windowbackground); n++;
  XtSetArg(args[n], XmNborderColor, windowborder); n++;
#endif /*COLORMESS*/
  if (buttonFont){ XtSetArg(args[n], XmNfontList, fontList); n++;}
  ncommon = n; /*use these args for all widgets in menus group */
  XtSetArg(args[n], XmNrowColumnType, XmMENU_BAR); n++;
  XtSetArg(args[n], XmNheight, MHEIGHT/2); n++;
  XtSetArg(args[n], XmNpacking, XmPACK_COLUMN); n++;
  XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
  XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
  XtSetArg(args[n], XmNrightOffset, BWIDTH+ZWIDTH); n++;
  XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
  menuBar = XmCreateMenuBar(grafwindow, "menuBar", args, n);
  XtManageChild(menuBar);

/*FILEMENU**********************************************************/
/*create a Pulldown Menu Pane */
  n = ncommon;
  fileMenu = XmCreatePulldownMenu(menuBar, "File", args, n);
  /*do not managechild here, popup level is not complete*/
/*create menu CascadeButton for this Menu Pane*/
  n = ncommon;
  XtSetArg(args[n], XmNsubMenuId, fileMenu); n++;
  fileButton = XmCreateCascadeButton(menuBar, "File", args, n);
  XtManageChild(fileButton);

/*create buttons on this Pulldown Menu Pane*/
/*this works for created colormap, fails for default colormap*/
  n = ncommon;
  openItem = XmCreatePushButton(fileMenu,"Open File...",args,n);
  XtAddCallback(openItem, XmNactivateCallback, open_CB, NULL); 
  XtManageChild(openItem);
  n = ncommon;
  aboutItem = XmCreatePushButtonGadget(fileMenu,"About MAGE",args,n);
  XtAddCallback(aboutItem, XmNactivateCallback, alert_CB, NULL); 
  XtManageChild(aboutItem);

/*    FileMenu sub: WriteMenu  **********/

/*create Pulldown MenuPane for Write Menu under File Menu*/
  n = ncommon;
  writeMenu = XmCreatePulldownMenu(fileMenu, "Write Menu dummy", args, n);
  /*do not managechild here, popup level is not complete*/

/*create menus CascadeButton for each MenuPane*/
  n = ncommon;
  XtSetArg(args[n], XmNsubMenuId, writeMenu); n++;
  writeButton = XmCreateCascadeButton(fileMenu, "Write Menu", args, n);
  XtManageChild(writeButton);
  n = ncommon;
  writeaboutItem = XmCreatePushButtonGadget(writeMenu,"About Write",args,n);
  XtAddCallback(writeaboutItem, XmNactivateCallback, writeabout_CB, NULL);
  XtManageChild(writeaboutItem);
  n = ncommon;
  writeviewItem = XmCreatePushButtonGadget(writeMenu,"Current View",args,n);
  XtAddCallback(writeviewItem, XmNactivateCallback, dummy_CB, NULL);
  XtManageChild(writeviewItem);
  n = ncommon;
  writefocusItem = XmCreatePushButtonGadget(writeMenu,"XYZ focus pt.",args,n);
  XtAddCallback(writefocusItem, XmNactivateCallback, dummy_CB, NULL);
  XtManageChild(writefocusItem);
  n = ncommon;
  writedrawnewItem =XmCreatePushButtonGadget(writeMenu,"Draw_new parts",args,n);
  XtAddCallback(writedrawnewItem, XmNactivateCallback, dummy_CB, NULL);
  XtManageChild(writedrawnewItem);
  n = ncommon;
  writekinItem = XmCreatePushButtonGadget(writeMenu,"modified kin",args,n);
  XtAddCallback(writekinItem, XmNactivateCallback, dummy_CB, NULL);
  XtManageChild(writekinItem);
  n = ncommon;
  writerottedItem = XmCreatePushButtonGadget(writeMenu,"rotated parts",args,n);
  XtAddCallback(writerottedItem, XmNactivateCallback, dummy_CB, NULL);
  XtManageChild(writerottedItem);
  n = ncommon;
  writePostScriptItem =
                    XmCreatePushButtonGadget(writeMenu,"PostScript...",args,n);
  XtAddCallback(writePostScriptItem, XmNactivateCallback, dummy_CB, NULL);
  XtManageChild(writePostScriptItem);
  n = ncommon;
  writeQAItem = XmCreatePushButtonGadget(writeMenu,"Questions & Ans",args,n);
  XtAddCallback(writeQAItem, XmNactivateCallback, dummy_CB, NULL);
  XtManageChild(writeQAItem);

/*    FileMenu sub: WriteMenu  _______________*/
  n = ncommon;
  quitItem = XmCreatePushButtonGadget(fileMenu,"Quit",args,n);
  XtAddCallback(quitItem, XmNactivateCallback, quit_CB, NULL);
  XtManageChild(quitItem);
/*FILEMENU______________________________________________________*/

/*EDITMENU*******************************************************/
/*create a Pulldown Menu Pane  */
  n = ncommon;
  editMenu = XmCreatePulldownMenu(menuBar, "Edit", args, n);
/*create CascadeButton for this Menu Pane*/
  n = ncommon;
  XtSetArg(args[n], XmNsubMenuId, editMenu); n++;
  editButton = XmCreateCascadeButton(menuBar, "Edit", args, n);
  XtManageChild(editButton);
  n = ncommon;
  editaboutItem = XmCreatePushButtonGadget(editMenu,"About Editing",args,n);
  XtAddCallback(editaboutItem, XmNactivateCallback, editabout_CB, NULL);
  XtManageChild(editaboutItem);
  n = ncommon;
  editviewItem = XmCreatePushButtonGadget(editMenu,"Update View...",args,n);
  XtAddCallback(editviewItem, XmNactivateCallback, dummy_CB, NULL);
  XtManageChild(editviewItem);
  n = ncommon;  /*Toggle buttons all have same args */
  XtSetArg(args[n], XmNindicatorOn, True); n++;
  XtSetArg(args[n], XmNfillOnSelect, True); n++;
  XtSetArg(args[n], XmNselectColor, (Pixel)1); n++;
  drawlineItem = XmCreateToggleButtonGadget(editMenu,"draw new...",args,n);
  XtAddCallback(drawlineItem, XmNvalueChangedCallback, dummy_CB, NULL);
  XtManageChild(drawlineItem);
  superpickItem = XmCreateToggleButtonGadget(editMenu,"superpick",args,n);
  XtAddCallback(superpickItem, XmNvalueChangedCallback, dummy_CB, NULL);
  XtManageChild(superpickItem);
  pickshowItem = XmCreateToggleButtonGadget(editMenu,"pickshow",args,n);
  XtAddCallback(pickshowItem, XmNvalueChangedCallback, dummy_CB, NULL);
  XtManageChild(pickshowItem);
  pruneItem = XmCreateToggleButtonGadget(editMenu,"prune",args,n);
  XtAddCallback(pruneItem, XmNvalueChangedCallback, dummy_CB, NULL);
  XtManageChild(pruneItem);

/*EDITMENU_______________________________________________________*/

/*WINDMENU*******************************************************/
/*create a Pulldown Menu Pane  */
  n = ncommon;
  windMenu = XmCreatePulldownMenu(menuBar, "Windows", args, n);
/*create CascadeButton for this Menu Pane*/
  n = ncommon;
  XtSetArg(args[n], XmNsubMenuId, windMenu); n++;
  windButton = XmCreateCascadeButton(menuBar, "Windows", args, n);
  XtManageChild(windButton);
  n = ncommon;
  showtextItem = XmCreatePushButtonGadget(windMenu,"Show text",args,n);
  XtAddCallback(showtextItem, XmNactivateCallback, dummy_CB, NULL);
  XtManageChild(showtextItem);
  n = ncommon;
  showcaptionItem = XmCreatePushButtonGadget(windMenu,"Show caption",args,n);
  XtAddCallback(showcaptionItem, XmNactivateCallback, dummy_CB, NULL);
  XtManageChild(showcaptionItem);
  n = ncommon;
  showgraphicsItem = XmCreatePushButtonGadget(windMenu,"Show graphics",args,n);
  XtAddCallback(showgraphicsItem, XmNactivateCallback, dummy_CB, NULL);
  XtManageChild(showgraphicsItem);
/*WINDMENU__________________________________________________________*/

/*KINEMENU***********************************************************/
/*create a Pulldown Menu Pane  */
  n = ncommon;
  kineMenu = XmCreatePulldownMenu(menuBar, "KINEMAGE", args, n);
/*create CascadeButton for this Menu Pane*/
  n = ncommon;
  XtSetArg(args[n], XmNsubMenuId, kineMenu); n++;
  kineButton = XmCreateCascadeButton(menuBar, "KINEMAGE", args, n);
  XtManageChild(kineButton);
  n = ncommon;
  nextItem = XmCreatePushButtonGadget(kineMenu,"Next",args,n);
  XtAddCallback(nextItem, XmNactivateCallback, next_CB, NULL); 
  XtManageChild(nextItem);
  n = ncommon;
  chooseItem = XmCreatePushButtonGadget(kineMenu,"Choose...",args,n);
  XtAddCallback(chooseItem, XmNactivateCallback, choose_CB, NULL);
  XtManageChild(chooseItem);
/*KINEMENU__________________________________________________________*/

/*OPTIMENU***********************************************************/
/*create a Pulldown Menu Pane */
  n = ncommon;
  optiMenu = XmCreatePulldownMenu(menuBar, "Options", args, n);
/*create a CascadeButton for this Menu Pane*/
  n = ncommon;
  XtSetArg(args[n], XmNsubMenuId, optiMenu); n++;
  optiButton = XmCreateCascadeButton(menuBar, "Options", args, n);
  XtManageChild(optiButton);
  n = ncommon;
  optionsaboutItem = XmCreatePushButtonGadget(optiMenu,"About Options",args,n);
  XtAddCallback(optionsaboutItem, XmNactivateCallback, optionsabout_CB, NULL); 
  XtManageChild(optionsaboutItem);
  n = ncommon;  /*Toggle buttons all have same args */
  XtSetArg(args[n], XmNindicatorOn, True); n++;
  XtSetArg(args[n], XmNfillOnSelect, True); n++;
  XtSetArg(args[n], XmNselectColor, (Pixel)1); n++;
  widthItem = XmCreateToggleButtonGadget(optiMenu,"one width",args,n);
  XtAddCallback(widthItem, XmNvalueChangedCallback, width_CB, NULL); 
  XtManageChild(widthItem);
  thinItem = XmCreateToggleButtonGadget(optiMenu,"thin line",args,n);
  XtAddCallback(thinItem, XmNvalueChangedCallback, thin_CB, NULL); 
  XtManageChild(thinItem);
  perspecItem = XmCreateToggleButtonGadget(optiMenu,"perspective",args,n);
  XtAddCallback(perspecItem, XmNvalueChangedCallback, perspec_CB, NULL); 
  XtManageChild(perspecItem);
  whtbkgItem = XmCreateToggleButtonGadget(optiMenu,"white bkg",args,n);
  XtAddCallback(whtbkgItem, XmNvalueChangedCallback, whtbkg_CB, NULL); 
  XtManageChild(whtbkgItem);
  B_WItem = XmCreateToggleButtonGadget(optiMenu,"B-W Monitor",args,n);
  XtAddCallback(B_WItem, XmNvalueChangedCallback, B_W_CB, NULL); 
  XtManageChild(B_WItem);
/*OPTIMENU___________________________________________________________*/

/*VIEWMENU************************************************************/

/*create a Pulldown Menu Pane */
  n = ncommon;
  viewMenu = XmCreatePulldownMenu(menuBar, "VIEWS", args, n);
/*create menu CascadeButton for this Menu Pane*/
  n = ncommon;
  XtSetArg(args[n], XmNsubMenuId, viewMenu); n++;
  viewButton = XmCreateCascadeButton(menuBar, "VIEWS", args, n);
  XtManageChild(viewButton);
  n = ncommon; /*10 view Toggle buttons: all have same args*/
  XtSetArg(args[n], XmNindicatorOn, True); n++;
  XtSetArg(args[n], XmNfillOnSelect, True); n++;
  XtSetArg(args[n], XmNselectColor, (Pixel)1); n++;
  reset1Item = XmCreateToggleButtonGadget(viewMenu,"View1",args,n);
  XtAddCallback(reset1Item, XmNvalueChangedCallback,  view1_CB, NULL); 
  XtManageChild(reset1Item);
  reset2Item = XmCreateToggleButtonGadget(viewMenu,"View2",args,n);
  XtAddCallback(reset2Item, XmNvalueChangedCallback,  view2_CB, NULL); 
  XtManageChild(reset2Item);
  reset3Item = XmCreateToggleButtonGadget(viewMenu,"View3",args,n);
  XtAddCallback(reset3Item, XmNvalueChangedCallback,  view3_CB, NULL); 
  XtManageChild(reset3Item);
  reset4Item = XmCreateToggleButtonGadget(viewMenu,"View4",args,n);
  XtAddCallback(reset4Item, XmNvalueChangedCallback,  view4_CB, NULL); 
  XtManageChild(reset4Item);
  reset5Item = XmCreateToggleButtonGadget(viewMenu,"View5",args,n);
  XtAddCallback(reset5Item, XmNvalueChangedCallback,  view5_CB, NULL); 
  XtManageChild(reset5Item);
  reset6Item = XmCreateToggleButtonGadget(viewMenu,"View6",args,n);
  XtAddCallback(reset6Item, XmNvalueChangedCallback,  view6_CB, NULL); 
  XtManageChild(reset6Item);
  reset7Item = XmCreateToggleButtonGadget(viewMenu,"View7",args,n);
  XtAddCallback(reset7Item, XmNvalueChangedCallback,  view7_CB, NULL); 
  XtManageChild(reset7Item);
  reset8Item = XmCreateToggleButtonGadget(viewMenu,"View8",args,n);
  XtAddCallback(reset8Item, XmNvalueChangedCallback,  view8_CB, NULL); 
  XtManageChild(reset8Item);
  reset9Item = XmCreateToggleButtonGadget(viewMenu,"View9",args,n);
  XtAddCallback(reset9Item, XmNvalueChangedCallback,  view9_CB, NULL); 
  XtManageChild(reset9Item);
  resetRItem = XmCreateToggleButtonGadget(viewMenu,"Reader's view",args,n);
  XtAddCallback(resetRItem, XmNvalueChangedCallback,  viewR_CB, NULL); 
  XtManageChild(resetRItem);
/*VIEWMENU________________________________________________________*/

/*OTHERMENU********************************************************/
/*create a Pulldown Menu Pane */
  n = ncommon;
  otherMenu = XmCreatePulldownMenu(menuBar, "Other", args, n);
/*create menu CascadeButton for this Menu Pane*/
  n = ncommon;
  XtSetArg(args[n], XmNsubMenuId, otherMenu); n++;
  otherButton = XmCreateCascadeButton(menuBar, "Other", args, n);
  XtManageChild(otherButton);
  n = ncommon;
  otheraboutItem = XmCreatePushButtonGadget(otherMenu,"About Other",args,n);
  XtAddCallback(otheraboutItem, XmNactivateCallback, otherabout_CB, NULL); 
  XtManageChild(otheraboutItem);

  n = ncommon; /* set of toggle buttons all have the same args*/
  XtSetArg(args[n], XmNindicatorOn, True); n++;
  XtSetArg(args[n], XmNfillOnSelect, True); n++;
  XtSetArg(args[n], XmNselectColor, (Pixel)1); n++;
  stereoItem = XmCreateToggleButtonGadget(otherMenu,"stereo on",args,n);
  XtAddCallback(stereoItem, XmNvalueChangedCallback,  stereo_CB, NULL); 
  XtManageChild(stereoItem);
  compareItem = XmCreateToggleButtonGadget(otherMenu,"compare on",args,n);
  XtAddCallback(compareItem, XmNvalueChangedCallback,  compare_CB, NULL); 
  XtManageChild(compareItem);
  measureItem = XmCreateToggleButtonGadget(otherMenu,"measures",args,n);
  XtAddCallback(measureItem, XmNvalueChangedCallback,  measure_CB, NULL); 
  XtManageChild(measureItem);

  n = ncommon;
  searchItem = XmCreatePushButtonGadget(otherMenu,"Find ...",args,n);
  XtAddCallback(searchItem, XmNactivateCallback, search_CB, NULL); 
  XtManageChild(searchItem);
  n = ncommon;
  readersviewItem=
           XmCreatePushButtonGadget(otherMenu,"set reader's view",args,n);
  XtAddCallback(readersviewItem, XmNactivateCallback, readersview_CB, NULL); 
  XtManageChild(readersviewItem);
/*    OTHERMENU: sub  KLUDESMENU**************/
/*create Pulldown Menu Pane for Kluges Menu under Other Menu*/
  n = ncommon;
  kludgeMenu = XmCreatePulldownMenu(otherMenu, "Kluges Menu dummy", args, n);
  /*do not managechild here, popup level is not complete*/
/*create menu CascadeButton for this Menu Pane*/
  n = ncommon;
  XtSetArg(args[n], XmNsubMenuId, kludgeMenu); n++;
  kludgeButton = XmCreateCascadeButton(otherMenu, "Kluges Menu", args, n);
  XtManageChild(kludgeButton);
  n = ncommon;
  kludgesaboutItem = XmCreatePushButtonGadget(kludgeMenu,"About Kluges",args,n);
  XtAddCallback(kludgesaboutItem, XmNactivateCallback, kludgesabout_CB, NULL);
  XtManageChild(kludgesaboutItem);
  n = ncommon;
  GFontItem = XmCreatePushButtonGadget(kludgeMenu,"Graphics Fonts...",args,n);
  XtAddCallback(GFontItem, XmNactivateCallback, GFont_CB, NULL);
  XtManageChild(GFontItem);
  n = ncommon;
  sterangItem = XmCreatePushButtonGadget(kludgeMenu,"stereo angle...",args,n);
  XtAddCallback(sterangItem, XmNactivateCallback, sterang_CB, NULL);
  XtManageChild(sterangItem);
  n = ncommon; /* set of toggle buttons all have the same args*/
  XtSetArg(args[n], XmNindicatorOn, True); n++;
  XtSetArg(args[n], XmNfillOnSelect, True); n++;
  XtSetArg(args[n], XmNselectColor, (Pixel)1); n++;
  xyzpointItem = XmCreateToggleButtonGadget(kludgeMenu,"XYZ point",args,n);
  XtAddCallback(xyzpointItem, XmNvalueChangedCallback, xyzpoint_CB, NULL);
  XtManageChild(xyzpointItem);
  gnomonItem = XmCreateToggleButtonGadget(kludgeMenu,"gnomon",args,n);
  XtAddCallback(gnomonItem, XmNvalueChangedCallback, gnomon_CB, NULL);
  XtManageChild(gnomonItem);
  n = ncommon;
  brotangleItem = XmCreatePushButtonGadget(kludgeMenu,"init angles",args,n);
  XtAddCallback(brotangleItem, XmNactivateCallback, brotangle_CB, NULL);
  XtManageChild(brotangleItem);
  n = ncommon;
  countItem = XmCreatePushButtonGadget(kludgeMenu,"count...",args,n);
  XtAddCallback(countItem, XmNactivateCallback, count_CB, NULL);
  XtManageChild(countItem);
  n = ncommon; /* set of toggle buttons all have the same args*/
  XtSetArg(args[n], XmNindicatorOn, True); n++;
  XtSetArg(args[n], XmNfillOnSelect, True); n++;
  XtSetArg(args[n], XmNselectColor, (Pixel)1); n++;
  rateItem = XmCreateToggleButtonGadget(kludgeMenu,"rate",args,n);
  XtAddCallback(rateItem, XmNvalueChangedCallback, rate_CB, NULL);
  XtManageChild(rateItem);
  testItem = XmCreateToggleButtonGadget(kludgeMenu,"Debug test",args,n);
  XtAddCallback(testItem, XmNvalueChangedCallback, test_CB, NULL);
  XtManageChild(testItem);
  n = ncommon;
  secretItem = XmCreatePushButtonGadget(kludgeMenu,"secret spells...",args,n);
  XtAddCallback(secretItem, XmNactivateCallback, secret_CB, NULL);
  XtManageChild(secretItem);
  n = ncommon; /* toggle buttons all have the same args*/
  XtSetArg(args[n], XmNindicatorOn, True); n++;
  XtSetArg(args[n], XmNfillOnSelect, True); n++;
  XtSetArg(args[n], XmNselectColor, (Pixel)1); n++;
  plotonlyItem = XmCreateToggleButtonGadget(kludgeMenu,"kaleidoscope",args,n);
  XtAddCallback(plotonlyItem, XmNvalueChangedCallback, plotonly_CB, NULL);
  XtManageChild(plotonlyItem);
/*    OTHERMENU: sub  KLUGESMENU_____________*/
/*    OTHERMENU: sub  HELPMENU*************/
/*create Pulldown Menu Pane for Help Menu under Other Menu*/
  n = ncommon;
  helpMenu = XmCreatePulldownMenu(otherMenu, "Help Menu dummy", args, n);
  /*do not managechild here, popup level is not complete*/
/*create menu CascadeButton for this Menu Pane*/
  n = ncommon;
  XtSetArg(args[n], XmNsubMenuId, helpMenu); n++;
  helpButton = XmCreateCascadeButton(otherMenu, "Help Menu", args, n);
  XtManageChild(helpButton);
  n = ncommon;
  helpaboutItem = XmCreatePushButtonGadget(helpMenu,"About Other Help",args,n);
  XtAddCallback(helpaboutItem, XmNactivateCallback, helpabout_CB, NULL);
  XtManageChild(helpaboutItem);
  mousehelpItem = XmCreatePushButtonGadget(helpMenu,"mouse operations",args,n);
  XtAddCallback(mousehelpItem, XmNactivateCallback, mousehelp_CB, NULL);
  XtManageChild(mousehelpItem);
  resizehelpItem = XmCreatePushButtonGadget(helpMenu,"window resizing",args,n);
  XtAddCallback(resizehelpItem, XmNactivateCallback, resizehelp_CB, NULL);
  XtManageChild(resizehelpItem);
/*    OTHERMENU: sub  HELPMENU____________*/
/*OTHERMENU____________________________________________________________*/
/*___________________________MENU__________________________________________*/

/*create pointID and distance message place, for PEX use only*/
  n = 0;/* create the message place for pointID, dist, etc. */
#ifdef COLORMESS
  XtSetArg(args[n], XmNvisual, magevisual); n++;
  XtSetArg(args[n], XmNdepth, magedepth); n++;
  XtSetArg(args[n], XmNcolormap, colormap); n++;
  XtSetArg(args[n], XmNbackground, windowbackground); n++;
  XtSetArg(args[n], XmNborderColor, windowborder); n++;
#endif /*COLORMESS*/
  XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
  XtSetArg(args[n], XmNheight, MHEIGHT/2); n++;
  XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
  XtSetArg(args[n], XmNrightOffset, BWIDTH+ZWIDTH); n++;
  XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
  XtSetArg(args[n], XmNmarginHeight, 0); n++;
  XtSetArg(args[n], XmNmarginWidth, 5); n++;
  XtSetArg(args[n], XmNeditable, False); n++;
  XtSetArg(args[n], XmNautoShowCursorPosition, False); n++;
  XtSetArg(args[n], XmNcursorPositionVisible, False); n++;
  if (buttonFont) { XtSetArg(args[n], XmNfontList, fontList); n++; }
  message = XtCreateWidget("message", xmTextWidgetClass, grafwindow, args, n);
  XtManageChild(message);

/*create drawArea directly in grafwindow as parent*/
  n = 0;/* create the drawing area for PEX or X11 */
#ifdef COLORMESS
  XtSetArg(args[n], XmNvisual, magevisual); n++;
  XtSetArg(args[n], XmNdepth, magedepth); n++;
  XtSetArg(args[n], XmNcolormap, colormap); n++;
  XtSetArg(args[n], XmNbackground, windowbackground); n++;
  XtSetArg(args[n], XmNborderColor, windowborder); n++;
#endif /*COLORMESS*/
  XtSetArg(args[n], XmNleftAttachment, XmATTACH_FORM); n++;
  XtSetArg(args[n], XmNtopAttachment, XmATTACH_FORM); n++;
  XtSetArg(args[n], XmNtopOffset, MHEIGHT/2); n++;
  XtSetArg(args[n], XmNrightAttachment, XmATTACH_FORM); n++;
  XtSetArg(args[n], XmNrightOffset, BWIDTH+ZWIDTH); n++;
  XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
  XtSetArg(args[n], XmNbottomOffset, MHEIGHT/2); n++;
  drawArea = XtCreateWidget("drawArea", xmDrawingAreaWidgetClass,
				   grafwindow, args, n);
/* add exposure and input callbacks */
  XtAddCallback(drawArea, XmNexposeCallback, drawArea_CB, NULL);
  XtAddCallback(drawArea, XmNinputCallback, pick_CB, NULL); /*MUX_MAIN.c*/
  XtAddCallback(drawArea, XmNresizeCallback, resize_CB, NULL);
  XtAddEventHandler(drawArea, Button1MotionMask, False, motion_handler, NULL);
  /*motion_handler and pick_CB in MUX_MAIN.c for consistency of ___MAIN stuff*/

#ifdef UNIX_PEX
/* these event handlers take care of blanking and restoring the dial labels */
  XtAddEventHandler(drawArea, EnterWindowMask, False, enter_handler, NULL);
  XtAddEventHandler(drawArea, LeaveWindowMask, False, leave_handler, NULL);
#endif /*UNIX_PEX*/

  XtManageChild(drawArea);

/*create topcapt as a topLevel shell with mage topLevel as parent*/
 /*topcapt = XtCreateApplicationShell("CAPT",topLevelShellWidgetClass,NULL,0);*/
  n = 0;  /* window title is a regular string */
#ifdef COLORMESS
  XtSetArg(args[n], XmNvisual, magevisual); n++;
  XtSetArg(args[n], XmNdepth, magedepth); n++;
  XtSetArg(args[n], XmNcolormap, colormap); n++;
  XtSetArg(args[n], XmNbackground, windowbackground); n++;
  XtSetArg(args[n], XmNborderColor, windowborder); n++;
#endif /*COLORMESS*/
  XtSetArg(args[n], XmNtitle, "MAGE CAPTION"); n++;
  /*XtSetValues(toptext, args, n);*/
/*
  topcapt = XtAppCreateShell(NULL, 
                             "Ppm", 
                             topLevelShellWidgetClass,
                             dpy, 
                             args,n);
*/
  topcapt = XtCreatePopupShell(NULL,
                               topLevelShellWidgetClass,
                               topLevel,
                               args, n);

/*create captwindow*/
  n = 0;  /* window title is a regular string */
#ifdef COLORMESS
  XtSetArg(args[n], XmNvisual, magevisual); n++;
  XtSetArg(args[n], XmNdepth, magedepth); n++;
  XtSetArg(args[n], XmNcolormap, colormap); n++;
  XtSetArg(args[n], XmNbackground, windowbackground); n++;
  XtSetArg(args[n], XmNborderColor, windowborder); n++;
#endif /*COLORMESS*/
  XtSetArg(args[n], XmNtitle, "CAPTION"); n++;
  XtSetArg(args[n], XmNx, 0); n++;
  XtSetArg(args[n], XmNy, 600); n++;
  XtSetArg(args[n], XmNwidth, 600); n++;
  XtSetArg(args[n], XmNheight, 200); n++;
  XtSetArg(args[n], XmNeditable, False); n++;
  XtSetArg(args[n], XmNscrollHorizontal, False); n++;
  XtSetArg(args[n], XmNscrollingPolicy, XmAUTOMATIC); n++;
  XtSetArg(args[n], XmNeditable, False); n++;
  XtSetArg(args[n], XmNeditMode, XmMULTI_LINE_EDIT); n++;
  XtSetArg(args[n], XmNautoShowCursorPosition, False); n++;
  XtSetArg(args[n], XmNcursorPositionVisible, False); n++;
  XtSetArg(args[n], XmNwordWrap, True); n++;
  /* create the captwindow for MAGE CAPTION  */
  captwindow =
    XmCreateScrolledText(topcapt,"captwindow", args, n);
  XtManageChild(captwindow);

 /*create toptext as a topLevel shell with mage topLevel as parent*/
/*toptext = XtCreateApplicationShell("TEXT",topLevelShellWidgetClass,NULL,0);*/
  n = 0;  /* window title is a regular string */
#ifdef COLORMESS
  XtSetArg(args[n], XmNvisual, magevisual); n++;
  XtSetArg(args[n], XmNdepth, magedepth); n++;
  XtSetArg(args[n], XmNcolormap, colormap); n++;
  XtSetArg(args[n], XmNbackground, windowbackground); n++;
  XtSetArg(args[n], XmNborderColor, windowborder); n++;
#endif /*COLORMESS*/
  XtSetArg(args[n], XmNtitle, "MAGE TEXT"); n++;
  /*XtSetValues(toptext, args, n);*/
/*
  toptext = XtAppCreateShell(NULL, 
                             "Ppm", 
                             topLevelShellWidgetClass,
                             dpy, 
                             args,n);
*/
  toptext = XtCreatePopupShell(NULL,
                               topLevelShellWidgetClass,
                               topLevel,
                               args, n);

/*create textwindow*/
  n = 0;  /* window title is a regular string */
#ifdef COLORMESS
  XtSetArg(args[n], XmNvisual, magevisual); n++;
  XtSetArg(args[n], XmNdepth, magedepth); n++;
  XtSetArg(args[n], XmNcolormap, colormap); n++;
  XtSetArg(args[n], XmNbackground, windowbackground); n++;
  XtSetArg(args[n], XmNborderColor, windowborder); n++;
#endif /*COLORMESS*/
  XtSetArg(args[n], XmNtitle, "TEXT"); n++;
  XtSetArg(args[n], XmNx, 0); n++;
  XtSetArg(args[n], XmNy, 20); n++;
  XtSetArg(args[n], XmNwidth, 600); n++;
  XtSetArg(args[n], XmNheight, 400); n++;
  XtSetArg(args[n], XmNscrollHorizontal, False); n++;
  XtSetArg(args[n], XmNscrollingPolicy, XmAUTOMATIC); n++;
  XtSetArg(args[n], XmNeditable, False); n++;
  XtSetArg(args[n], XmNeditMode, XmMULTI_LINE_EDIT); n++;
  XtSetArg(args[n], XmNautoShowCursorPosition, False); n++;
  XtSetArg(args[n], XmNcursorPositionVisible, False); n++;
  XtSetArg(args[n], XmNwordWrap, True); n++;
  /* create the textwindow for MAGE TEXT  */
  textwindow =
    XmCreateScrolledText(toptext,"textwindow", args, n);
  XtManageChild(textwindow);

  XtPopup(topgraf,XtGrabNone);
  XtRealizeWidget(topgraf);    /* realize widget hierarchy */
  XtPopup(topcapt,XtGrabNone);
  XtRealizeWidget(topcapt);    /* realize widget hierarchy */
  XtPopup(toptext,XtGrabNone);
  XtRealizeWidget(toptext);    /* realize widget hierarchy */

  drawWindow = XtWindow(drawArea);	/* get the d.a. widget's window ID*/

#ifdef SHORTWAY
  dpy = XtDisplay(drawArea); /* get the display connection */
#endif /*SHORTWAY*/

#ifndef UNIX_PEX  /*GENERIC UNIX uses offscreen buffer space */

#endif /*of NOT UNIX_PEX*/
/*create offscreenpixmap*/
   offscreen = XCreatePixmap(dpy,RootWindow(dpy,DefaultScreen(dpy))
                               ,GWIDTH,GHEIGHT,magedepth);

   gc = XCreateGC(dpy,drawWindow,0L,NULL);
   XSetForeground(dpy,gc,WhitePixel(dpy,DefaultScreen(dpy)));
   XSetBackground(dpy,gc,BlackPixel(dpy,DefaultScreen(dpy)));

   offgc = XCreateGC(dpy,offscreen,0L,NULL);
   XSetForeground(dpy,offgc,WhitePixel(dpy,DefaultScreen(dpy)));
   XSetBackground(dpy,offgc,BlackPixel(dpy,DefaultScreen(dpy)));

/*Generic Mage: zoom, zslab, and ztrans  are done with on-screen sliders */
   creategrafzoomBar();  /*___BAR.c*/
   creategrafzclipBar(); /*___BAR.c*/
   creategrafztranBar(); /*___BAR.c*/

/*finished mainsetup, set flag so mage will do beginnings() */
   Lbegin = 1;
}
/*___mainsetup()____________________________________________________________*/

/****definecursors()*********************************************************/
void    definecursors(){;} 
/*___definecursors()________________________________________________________*/

/****beginnings()*************************************************************/
void    beginnings()
{
    int        j,launched;
    
        launched = 0;
        Lbegin = 0;
        fp = NULL;
        IOerr = 1;    /* Begins asumming no file */ 
            /* and various flags set for no legitimate file */

        launched = Lstartfile; /*gotten from ARGC in MAIN */

        makecursoractive(); /*MACMAIN*/
        /*initial dialog box here for always initial box*/
        setMageinitDLOGreport(); /*message*/  /*MAGEINIT.C*/
        DoMageinitDLOGreport(); /*now can set MAXNUM & MAXDRAWS */
        
      /*  XRaiseWindow(dpy,textwindow);*/ /* next thing is a message*/
             /* to be shown in the textWindow.  Avoids flipping to graphics */
             /* and messing with the graphics cursor choices */
        
        if( launched != 0) /* invoked by a file */
        {
            makecursorwait();  /*MUX_MAIN.C*/
            IOerr = 0; /* putative .kin file available */
            Lnewfile = 1;
            Lstartfile = 1;
            sprintf(word,"%s\n A request for a kinemage "      /*c*/
                         "file has launched MAGE.",version);
            cleartexts();      /*MUXMWIND.C*/
            inserttexts(word); /*MUXMWIND.C*/
        }
        else
        {
            makecursoractive();  /*MACMAIN.C*/
           /*initial dialog box here for no box on file launch*/

           sprintf(word,"%s\nUse File pull-down menu above"    /*c*/ 
                        " to Open a kinemage File.",version);

            cleartexts();         /*MUXMWIND.C*/
            inserttexts(word);    /*MUXMWIND.C*/
        }

/*3456789_123456789_123456789_123456789_123456789_123456789_123456789_12345678*/
        /*MAGEINIT.C  alloc storage for big arrays*//*do this before open file*/
        allocdataarrays(0);
          /* *fxyz[3] array of float pointers, fxyz[3][MAXNUM]*/
          /* *xyz[4] array of int pointers,xyz[4][MAXNUM]*/
          /* *com  pointer to char array, com[MAXNUM][16]*/
          /*  int    inwordcom[MAXNUM];  */   /* size = 2*MAXNUM */
          /* index into the comment or wordlist character array  */
        allocdisplayarrays(0);  /* *it[8]   array of pointers, it[8][MAXDRAWS]*/
        allocwordcomarray(0); 
          /* char  *wordcom;  pointer to array, for wordcom[MAXWORDCOM], */
          /*size = MAXWORDCOM*/ /* the comment or wordlist character array  */
}
/*___beginnings()____________________________________________________________*/

