      /*MUXPHIG.c*/ 
/* 
       Once upon a time:
       This file contained the PHIGS specific routines for the
       example program4,  phigs4.c  that was a demo program
       came with the ESV. It has been hacked beyond almost all
       recognition, but thanks are due to:
       Author: James Buckmiller  May 1990, Evans & Sutherland.
       Modified:  J. Buckmiller  Mar 1991.   Approved C binding 
          Much of the setup used here derives directly from that
       code.
            Dave Richardson, June 1994
*/

#include <X11/phigs/phigs.h>
#include <phigs/esgdp.h>
#include <X11/extensions/XPick.h>
#include <X11/extensions/XInput.h>
#include <X11/phigs/esphigs.h>
#include "MAGE.H"

#define WS             1
#define DISPLAY_STRUCT 1000
#define LOCAL          5
#define VIEW           6
#define TRANS          7
#define SPACEKEYSYM   32
#define MAXDIALS       8                /* number of dials on dials box */ 
#define CHARS_PER_DIAL 8                /* size of dials labels */ 
#define SAFE_PEX(routine) Error_Check(__FILE__, __LINE__, routine)
#define DEG_TO_RAD(D)   ((3.14159265358 / 180.0) * (D))
#define DIALSCALE     .001 /*.005*/ /* Dial scale value */
#define PICKABLE      10                /* name for inclusion filter */

#define EXTURN 
  EXTURN int Major_op;                    /* input extension values */
  EXTURN int First_ev;
  EXTURN int First_err;

  EXTURN Pint_list nset_names;            /* Pick list nameset declaration */
  EXTURN XPickFilter pick_incl, pick_excl; /* Pick filters declaration      */
  EXTURN Pint namesints[1];               /* Name arrays for nameset       */
  EXTURN PC pc;                           /* pick context                  */
  EXTURN Pint PEX_error;                  /* For PEX error numbers */
  EXTURN Pmatrix3 identity;
  EXTURN  int     DeviceMotion;           /* device motion event type */
  EXTURN  Pint    PEX_error;              /* For PEX error numbers */

  EXTURN  int Major_op, First_ev, First_err;  /* pick extension parameters */
  EXTURN  Pint_list nset_names;               /* pick list nameset */
  EXTURN  Pint namesints[1];                      /* nameset list arrays */
  EXTURN  XPickFilter pick_incl, pick_excl; /*pick inclusion/exclusion filters*/
  EXTURN  PC pc;                                  /* pick context */
  EXTURN  KeySym knob_labels[MAXDIALS][CHARS_PER_DIAL];

  /*mage stuff below here */
  EXTURN int      knob_totals[8],knob_hold[8];
  EXTURN Ppoint3  pxyz[23500];
  EXTURN Ppoint_list3 pxyz_list[23500];
  EXTURN Pixel   icolorindex[256];
  EXTURN XDevice *dials;           /* dials device */
  EXTURN Boolean done;
  EXTURN Ppoint3  cntr,cntrlocal;
  EXTURN Pvec3 scalep;
  EXTURN Pvec3 scalelocal;
  EXTURN Pvec3 trans;
  EXTURN Pvec3 translocal;
  EXTURN float rotx, roty, rotz;
  EXTURN Pmatrix3 currmatrix;
  EXTURN Pmatrix3 centermatrix;
  EXTURN Pmatrix3 mx;
  EXTURN Pmatrix3 my;
  EXTURN Pmatrix3 mz;
  EXTURN Pmatrix3 tempmatrix;
  EXTURN Pmatrix3 holdmatrix;
  EXTURN Pmatrix3 rotonlymatrix;
  EXTURN Pdcue_bundle dcuebundle;
  EXTURN Pview_rep3  vrep;        /*Declare viewport variable*/

/*SUBROUTINE*PROTOTYPES************/
void Error_Check(char *, int, char *);
void setviewingconditions(void);
void SetupWorkstation(Display *, Window);      
void makepexstructures(void);
void updatevisibility(void);
void make_identity(Pmatrix3);
void make_viewmatrix(Pmatrix3);
void Cleanup(void); 
void makepexcolours(void); 
void resetalltransformations(void);
void resetpex(void);
void setpexmeasures();
void enter_handler(Widget, caddr_t, XEvent *,Boolean *);
void leave_handler(Widget, caddr_t, XEvent *,Boolean *);
/*     Widget widget;                 */
/*     caddr_t client_data;           */
/*     XEvent *event;                 */
/*     Boolean *continue_to_dispatch; */
void open_knob(void);
void init_pick(void);
void knobmotionhandler(void);
void buttonscallback(Widget, XtPointer , XtPointer);
void pickeventhandler(int,float,float,float);
void OpenPex(Display *);
void OpenPhigs(void);
void dospecialevent(XEvent *); /*MUXPHIG.c*/
void domotionevent(XEvent *);  /*MUXPHIG.c*/
void dopickingevent(XEvent *); /*MUXPHIG.c*/
void doxpick(Widget, int, int);
void doxprepick(Widget, int, int);
void doxpickpredraw(); /*MUXPHIG.c*/
void setpexclipplanes();       /*MUXPHIG.c*/

/****Error_Check()************************************************************/
void Error_Check(char *File, int Line, char *Routine)
{
    if (PEX_error) /*global variable stores PEX error codes */
    {
      printf("(file %s; line %d):\n", File, Line);
      printf("\t?unexpected PEX error %d in routine %s\n",
              PEX_error, Routine);
      exit(1);
    }
}
/*___Error_Check()___________________________________________________________*/


/****setviewingconditions()***************************************************/
void    setviewingconditions()
{
  /*Pview_rep3       vrep;*/      /* Declare viewporting variables */
  Pview_map3       map;
  Ppoint3          vrp, cntr;
  Pvec3            vup;
  Pvec3            vpn;

  /* Setup viewport parameters */

  map.proj_type = PTYPE_PARAL;                        /* Set projection type */
  map.vp.x_min = 0.0; map.vp.x_max = 1.0; /* Set viewport limits */
  map.vp.y_min = 0.0; map.vp.y_max = 1.0; /* these have to be 0 <=  .  <= 1 */
  /*map.vp.z_min = 0.0; map.vp.z_max = 1.0;*/
  if(izclip > 200) izclip = 200;
  if(izclip < 0  ) izclip = 0;
  /*vrep limits must be 0.0 <= . <= 1.0 */
  map.vp.z_min = 0.5 - (float)izclip/400.0;
  map.vp.z_max = 0.5 + (float)izclip/400.0;


  map.win.x_min = -1.5; map.win.x_max = 1.5; /* Set window limits   */
  map.win.y_min = -1.5; map.win.y_max = 1.5;

  /* Set the front and back planes that define the viewing volume*/
  map.back_plane  = -3.0;  /* Set the front and back planes */
  map.front_plane =  1.0; 
  map.view_plane  =  0.0;    /* Set the location of the view plane     */

  map.proj_ref_point.x = 0.0;           /* Set projection Reference point */
  map.proj_ref_point.y = 0.0;           /* in VRC space                   */
  map.proj_ref_point.z = 3.0;
                            
  vrep.xy_clip    = PIND_NO_CLIP;    /* Turn Viewport clipping off           */
  /*vrep.back_clip  = PIND_NO_CLIP;*/ /*not to be confused with the clipping */
  /*vrep.front_clip = PIND_NO_CLIP;*/    /* at the viewplanes! */
  if(Lzclipon)
      vrep.back_clip = vrep.front_clip = PIND_CLIP;
  else
      vrep.back_clip = vrep.front_clip = PIND_NO_CLIP;


  vrep.clip_limit = map.vp;     /* Set Viewport clipping volume = viewport */

  /* Setup View Reference Coordinates */

  vrp.x = 0.0; vrp.y = 0.0; vrp.z = 1.0;             /* Set View ref point   */
  /* Set view up vector   */
  vup.delta_x = 0.0; vup.delta_y = 1.0; vup.delta_z = 0.0;
  /* Set view plane normal*/
  vpn.delta_x = 0.0; vpn.delta_y = 0.0; vpn.delta_z = 1.0;

  peval_view_ori_matrix3(&vrp, &vpn, &vup,        /* Evaluate orient matrix */
                 &PEX_error, vrep.ori_matrix);

  SAFE_PEX("peval_view_ori_matrix3");             /* Check for error status */

  peval_view_map_matrix3( &map, &PEX_error,        /* Evaluate map matrix    */
                           vrep.map_matrix);

  SAFE_PEX("peval_view_map_matrix3");              /* Check for error status */

  pset_view_rep3( WS, VIEW, &vrep );         /* Set the view representation  */

  /*set depth cue parameters*/
  dcuebundle.mode = 1; /*mode 1 allows depth cueing*/
  dcuebundle.ref_planes[0] =  0.0; /*the back  plane, orig 0.0*/
  dcuebundle.ref_planes[1] =  1.0; /*the front plane, orig 1.0*/
  dcuebundle.scaling[0] = 0.2; /*scaling at the back  plane, orig 0.0*/
  dcuebundle.scaling[1] = 1.0; /*scaling at the front plane, orig 1.0*/
  dcuebundle.colr.type = PINDIRECT; /*way to find background fade-to color*/
  dcuebundle.colr.val.ind  = 0; /*index to the background fade-to color*/

  pset_dcue_rep(WS, 2, &dcuebundle);


}
/*___setviewingconditions()_________________________________________________*/


/*                       
   SetupWorkstation

   This routine opens a PHIGS workstation and sets up a Viewport.
   The structure edit mode is set to insert elements and the display
   update state is set to PWAIT. 
   Z buffering is enabled by calling psethlhsrmode
   The nameset list is then filled with the name PICKABLE. This list is
   used to set the picking inclusion filter. This same list will be used
   in the addnameset call in makepexstructures() to add the PICKABLE name 
   to the structure. The UNPICKABLE name is listed in the exclusion filter but
   is not included in any display structures, thus nothing is excluded from
   being picked.  However, once picked, a structure is scanned to see if it
   has any MAGE set flags that would disallow it being picked.
*/
/****SetupWorkstation()*******************************************************/
void SetupWorkstation(dpy, win)      
Display *dpy;
Window   win;
{                          
  Pconnid_x_drawable  conn;    
 

  conn.display     = dpy;
  conn.drawable_id = win;

  popen_ws(WS, (Pconnid *) (&conn), phigs_ws_type_x_drawable);  /* Open WS */

  izclip = 200; /*in case these haven't been set yet to default values*/
  Lzclipon = 1;
  setviewingconditions();

  pset_edit_mode(PEDIT_INSERT);          /* Set edit mode to insert elements */
  pset_disp_upd_st(WS, PDEFER_WAIT, PMODE_NIVE); /* Set update state to WAIT */

  pset_hlhsr_mode(1, PHIGS_HLHSR_MODE_ZBUFF);  /* Enable WS Z buffering     */

  nset_names.num_ints = 1;                     /* Name set list count       */
  nset_names.ints = namesints;                 /* Name set list of integers */
  namesints[0] = PICKABLE;                     /* Put the PICKABLE name in  */
                                               /*  the name set list.       */


  pick_incl.number = 1;           /* Set Picking inclusion filter to 1 name  */
  pick_incl.integers = namesints; /* Set the names list to be same as nameset*/

  pick_excl.number = 0;           /* Empty picking exclusion filter          */
  pick_excl.integers = NULL;

  XCreatePC(dpy, pick_PEX, &pc);                 /* Create picking context */
  XSetPickFilters(dpy, pc, &pick_incl, &pick_excl);  /* Set pick filters   */



}
/*___SetupWorkstation()______________________________________________________*/

/****makepexstructures()*****************************************************/
void makepexstructures()
{
Pmatrix3 identity;
int k,i,grpmin,grpmax,setmin,setmax,jline,j,jmin,jmax,Lpolyline;
int maximumgroup, kk,iquit;
  Display *dpy = XtDisplay(drawArea);
  int      scr = DefaultScreen(dpy);
  Colormap cmap = DefaultColormap(dpy,scr);
  XColor  testcolor;

Pvec3   aoffset[2];

Pint       index;
Pinq_type  type;
Pcolr_rep  *rep;
Pgse_sphere_div_data sphere_div;  /*940623*/
Pgse_sphere_radius_data sphere_radius;  /*940622*/
Pgse_data  gse_struct;                  /*940622*/
/*Pgdp_data3 *gdp_data;*/
/*Pgdp_data3 gdp_data;*/
Pdata gse_data, gdp_data; 
   /*Vicki Tumia, 940627 this definition uses just Pdata for both*/
Pgse_sphere_div_data *division; 
   /*division structure holds division precision information*/
Pgse_sphere_radius_data *radius;
   /*radius structure holds radius value for a ball list*/
Psphere_data *sd; /*holds color model and number of spheres*/
Ppoint_list3 pt_lst; /*point list is a dummy for generalized sphere drawer*/
Psphere *sp; /*center x,y,z points */



Psphere_w_radius_colr  radcolr_sphere;

Pint       *error_ind;

/*  point vectors */
/*fxyz is here the raw data, points has to be fully loaded in sequence*/
for(i=1;i<=maxnxyz;i++)
{
    pxyz[i].x = fxyz(0,i);
    pxyz[i].y = fxyz(1,i);
    pxyz[i].z = fxyz(2,i);
/*
printf("pxyz[%d] %f, %f, %f; xyz[%d] %d, %d, %d, %d\n"
,i,pxyz[i].x,pxyz[i].y,pxyz[i].z
,i,xyz(0,i),xyz(1,i),xyz(2,i),xyz(3,i));
*/
}

 pset_edit_mode(PEDIT_INSERT);       /* Set edit mode to insert elements    */
 punpost_all_structs(WS);            /* Unpost to remove any old structures */
 pdel_all_structs();                 /* Delete any old structures           */
 /*set up to create structures for each Mage list */
 if(Lmarkers)
 {
      maximumgroup = maxgrp+1; /* maxgrp+1 does markers */
      if(Lmeasures) maximumgroup = maxgrp+2; /* maxgrp+2 does measures */
      if(Ldrawline) maximumgroup = maxgrp+3; /* maxgrp+3 does drawline */
 }
 else maximumgroup = maxgrp;

 for(ngrp = 1; ngrp <= maximumgroup ; ngrp++)
 {/*loop over groups*/
    grpmin = igrp[ngrp][1];
    grpmax = igrp[ngrp][2];
    for(nset = grpmin; nset <= grpmax; nset++)
    {/*loop over subgroups*/
        setmin = iset[nset][1];
        setmax = iset[nset][2];
        for(nlist = setmin; nlist <= setmax ; nlist++)
        {/*loop over lists*/
            /* Every list can define a phigs structure */
            Lnew = 1; /*get ready to define a structure*/
            jmin = ilist[nlist][1];
            jmax = ilist[nlist][2];
            Lpolyline = 0; /*expect a line start*/
            for( j = jmin ; j <= jmax ; j++)
            {/*loop over input points*/
                if(xyz(0,j) < 10) /*drawto type point on a line */
                {/*continuing a polyline list*/
                    ; 
                }
                else /*anything closes a polyline if one is open*/
                {/*begin something new*/
                    if(Lnew)
                    {/*create a structure for this list*/
/*
printf("Lnew==1, about to open a phigs structure: %d\n",nlist);
*/
                        popen_struct((Pint)nlist);
                           /*index phigs structures by list number*/
                        Lnew = 0;
/*
printf("Lnew==0, just opened a phigs structure\n");
*/
                    }
                    if(Lpolyline)
                    {/*create polyline list element for previous line*/
                        /* Assign this list's index color to lines*/
                        pset_line_colr_ind(ilist[nlist][3]+10);/*color*/
                        /* Fill in number of points in list */
                        pxyz_list[jline].num_points = j-jline; 
                        /* Pointer to point array at first point of polyline*/
                        pxyz_list[jline].points = pxyz+jline;
                        /* Create a polyline element */
                        ppolyline3(&pxyz_list[jline]);
                    }/*create polyline list element for previous line*/
                    Lpolyline = 0;
                    if(xyz(0,j) >= 10 && xyz(0,j) < 20 )/* moveto */
                    {/*start a polyline list */
                        /* Remember this j as the polyline start */
                        jline = j;
                        Lpolyline = 1; /*now in a polyline*/
                    }/*start a polyline list */
                    /*if(xyz(0,j) == 20||xyz(0,j) == 25 )*/ /*dot 20*/
                    if(xyz(0,j) == 20 ) /*dot 20*/
                    {/*set a phigs marker*/
                        /* Assign this list's index color to markers*/
                        pset_marker_colr_ind(ilist[nlist][3]+10); /*color*/
                        /*if(xyz(0,j) == 20 )*/ /*dot 20*/
                        /*  pset_marker_type(PMARKER_DOT);*/
                        /*if(xyz(0,j) == 25 )*/ /*ball 25*/
                          pset_marker_type(PMARKER_ASTERISK);
                        pset_marker_size(1.0);
                        /* Fill in number of points in list */
                        pxyz_list[j].num_points = 1; 
                        /*Pointer to point array at first point of polymarker*/
                        pxyz_list[j].points = pxyz+j;
                        /* Create a polymarker element */
                        ppolymarker3(&pxyz_list[j]);
                    }/*set a phigs marker*/
                               /*???*/
                    if(xyz(0,j) == 25 ) /* ball 25*/
                    {/*set a phigs ball*/
/*pset_hlhsr_id(PHIGS_HLHSR_ID_ON);*/ /*set zbuffering, needed for spheres??*/
/*Psphere_w_radius_colr  radcolr_sphere;*/
                        /* Assign this list's index color to balls*/
                        pset_int_colr_ind(ilist[nlist][3]+10); /*color*/
                        pset_line_colr_ind(ilist[nlist][3]+10);/*color*/
                        pset_int_style(PSTYLE_SOLID);
                           /* Set interior style to be solid*/
                        /*  set the precision of the sphere  940627*/
                        gse_data.size = (size_t) sizeof(Pgse_sphere_div_data);
                        gse_data.data = malloc ((int)gse_data.size);
                        division = (Pgse_sphere_div_data *)gse_data.data;
                        /*division->div = 10.0;*/
                        division->div = 3.0;
                        pgse(PES_GSE_SPHERE_DIVISIONS, &gse_data);

                        /*  set the radius of the sphere 940627 */
                        gse_data.size = (size_t)sizeof(Pgse_sphere_radius_data);
                        gse_data.data = malloc((int)gse_data.size);
                        radius = (Pgse_sphere_radius_data *)gse_data.data;
                        radius->radius = 0.2;
                        pgse(PES_GSE_SPHERE_RADIUS, &gse_data);

                        pxyz_list[j].num_points = 0; 
                        /*Pointer to point array at first point of spheres*/
                        /*pxyz_list[j].points = pxyz+j;*/
                        /* /usr/include/X11/phigs */


                        /*radcolr_sphere.head.colr_model = (Pint)PMODEL_RGB;*/
                       /* radcolr_sphere.head.colr_model = (Pint)PINDIRECT;*/
                       /* radcolr_sphere.head.count = (Pint)1;*/
                       /* radcolr_sphere.data[0].center.x = (Pfloat)pxyz[j].x;*/
                       /* radcolr_sphere.data[0].center.y = (Pfloat)pxyz[j].y;*/
                       /* radcolr_sphere.data[0].center.z = (Pfloat)pxyz[j].z;*/
                       /* radcolr_sphere.data[0].radius = (Pfloat)0.1;*/
                       /* radcolr_sphere.data[0].colr.ind = 
                              (Pint)(ilist[nlist][3]+10);*/ /*color*/
                       /* gdp_data.unsupp.data = 
                                  (char *)&radcolr_sphere;*/
                       /* gdp_data.unsupp.size = 
                                  (size_t)sz_Psphere_w_radius_colr(1);*/

                        /* Create a sphere element */
                       /* pgdp3(&pxyz_list[j]*/
                       /*       ,PES_GDP_SPHERE_RADIUS_COLR,&gdp_data);*/


                        /*define a list of simple spheres that contains */
                        /*  1 list of 1 sphere, ie, define 1 sphere  */
                        gdp_data.size = (size_t)(sizeof(Psphere_simple));
                        gdp_data.data = malloc((int)gdp_data.size);
                        sd = (Psphere_data *)gdp_data.data;
                        sd->count = 1;
                        sd->colr_model = PINDIRECT;
                        sp = (Psphere *)(sd+1);
                        sp->center.x = (Pfloat)pxyz[j].x;
                        sp->center.y = (Pfloat)pxyz[j].y;
                        sp->center.z = (Pfloat)pxyz[j].z;
                        pgdp3(&pxyz_list[j], PES_GDP_SPHERE, &gdp_data);




/*pset_hlhsr_id(PHIGS_HLHSR_ID_OFF);*/  /*set zbuffering, needed for spheres??*/
                    }/*set a phigs ball*/
                    /*-------------------------------------------*/
                    if(xyz(0,j) == 30 ) /*LABEL*/
                    {/*set a phigs label*/
                        /* Assign this list's index color to text*/
                        pset_text_colr_ind(ilist[nlist][3]+10); /*color*/
                        /*pset_text_font(PFONT_MONO);*/
                        pset_anno_char_ht(0.015); /*default==0.01*/
                        /*pset_anno_char_ht(0.03);*/ /*default==0.01*/
                        /*Pointer to point array of text == pxyz+j */
                       for(k=0; k<(MAXPTIDCHAR+1); k++)
                       {
                           icom = (long)(MAXPTIDCHAR+1)*(long)j + (long)k;
                           /*note: define  pcom1(icom)  *(com + icom)  */
                           /*word[k] = *( com1(icom) ); */
                           /*word[256] is a general use common variable */
                           word[k] = ( pcom1(icom) );
                        }
                        /* Create a text element */
                        aoffset[0].delta_x = 0.0;
                        aoffset[0].delta_y = 0.0;
                        aoffset[0].delta_z = 0.0;
                  
                        panno_text_rel3(pxyz+j,aoffset,word);
                    }/*set a phigs label*/
                    /*--------------------------------------------*/
                    if(xyz(0,j) == 40 ) /*WORDS*/
                    {/*set a phigs wordlist*/
                        /* Assign this list's index color to text*/
                        pset_text_colr_ind(ilist[nlist][3]+10); /*color*/
                       /* pset_text_font(PFONT_MONO);*/
                        pset_anno_char_ht(0.015); /*default==0.01*/
                        /*Pointer to point array of text == pxyz+j */
                       for(k=0;k<=255;k++)
                       {
                           commentstr[k] =
                               pwordcom( (long)pinwordcom( (long)j) +(long)k);
                           if(k>2 && commentstr[k]=='\0') break;
                       }
                        aoffset[0].delta_x = 0.0;
                        aoffset[0].delta_y = 0.0;
                        aoffset[0].delta_z = 0.0;
                       kk = 3;
                       iquit=0;
                       while(!iquit)
                       {
                         for(k=0; k<=(255-kk); k++)
                         {
                           word[k] = commentstr[k+kk];
                           if(word[k]=='\0'){word[k]=='\0';iquit=1;break;}
                           if(word[k]==EOL){word[k]=='\0';kk=kk+k+1;break;}
                         }
                         /* Create a text element */
printf("wordlist(part)[%d]: %s\n",k,word);
                         panno_text_rel3(pxyz+j,aoffset,word);
                         if(!iquit) aoffset[0].delta_y=aoffset[0].delta_y-.04;
                       }
                    }/*set a phigs wordslist*/
                    /*--------------------------------------------*/
                }/*begin something new*/
            }/*loop over input points*/
            if(Lnew)
            {/*create a structure for this list */
/*
printf("just finished with a list and about to open a phigs structure: %d\n",nlist);
*/
                popen_struct((Pint)nlist);
                    /*index phigs structures by list number*/
                Lnew = 0;
/*
printf("just finished with a list, just opened a phigs structure\n");
*/
            }
            if(Lpolyline)
            {/*create polyline list element for previous line*/
                /* Assign this list's index color to lines*/
                pset_line_colr_ind(ilist[nlist][3]+10);/*color*/
                /* Fill in number of points in list */
                pxyz_list[jline].num_points = j-jline; 
                /* Pointer to point array at first point of polyline*/
                pxyz_list[jline].points = pxyz+jline;
                /* Create a polyline element */
                ppolyline3(&pxyz_list[jline]);
            }
            Lpolyline = 0;
            /* Close this list's structure */
            if(Lnew==0)
            {
/*
printf("just finished a list and about to close a phigs structure\n");
*/
                 pclose_struct();
            }
        }/*loop over lists*/
    }/*loop over subgroups*/
 }/*loop over groups*/
      /*pset_int_style(PSTYLE_SOLID);*/ /* Set interior style to be solid*/
      /*pset_int_colr_ind(2);*/ /* Assign default index color 2 to face */
      /*pfill_area3(&Fill_list[2]);*/

 make_identity(identity);
 make_viewmatrix(currmatrix);

 /*???*/
 popen_struct(DISPLAY_STRUCT); /* Open the top level display structure */
     pset_view_ind(VIEW);           /* Set the view index to be used        */
     pset_hlhsr_id(PHIGS_HLHSR_ID_ON);     /* Turn on Z buffering           */
     pset_dcue_ind(2); /*allow depth cueing*/
     pset_linewidth(2.0); /*double width*/
     padd_names_set(&nset_names);   /* Add nameset for picking  (PICKABLE)  */
     plabel(LOCAL);                 /* Insert a label for future updates    */
     pset_local_tran3(identity, PTYPE_REPLACE);/*set transformation matrix*/
     plabel(TRANS);                 /* Insert a label for future updates    */
     pset_global_tran3(identity, PTYPE_REPLACE);/*set transformation matrix*/
 for(ngrp = 1; ngrp <= maximumgroup ; ngrp++)
 {/*loop over groups*/
    grpmin = igrp[ngrp][1];
    grpmax = igrp[ngrp][2];
    for(nset = grpmin; nset <= grpmax; nset++)
    {/*loop over subgroups*/
        setmin = iset[nset][1];
        setmax = iset[nset][2];
        for(nlist = setmin; nlist <= setmax ; nlist++)
        {/*loop over lists*/
            /* Every list defines a phigs structure */
            pexec_struct((Pint)nlist); 
        }/*loop over lists*/
    }/*loop over subgroups*/
 }/*loop over groups*/
 pset_hlhsr_id(PHIGS_HLHSR_ID_OFF);  /* Turn off Z buffering*/
 pclose_struct(); /*DISPLAY_STRUCT*/

 ppost_struct(WS, DISPLAY_STRUCT, 1.0); /* Post DISPLAY_STRUCT, prio 1 */

 pupd_ws(WS,PUPD_PERFORM);   /* Update the workstation      */
 pset_edit_mode(PEDIT_REPLACE);   /* Set edit mode to replace    */
 predraw_all_structs(WS, PFLAG_ALWAYS); /* Redraw the structure */
}
/*___makepexstructures()____________________________________________________*/

/****updatevisibility()******************************************************/
void updatevisibility()
{
int grpmin,grpmax,setmin,setmax;
int maximumgroup;
Pmatrix3 identity;


 pset_edit_mode(PEDIT_INSERT);       /* Set edit mode to insert elements    */

 punpost_all_structs(WS); /* Unpost to remove any old structures */
 pdel_struct(DISPLAY_STRUCT);
 popen_struct(DISPLAY_STRUCT); /* Open the top level display structure */
     pset_view_ind(VIEW);           /* Set the view index to be used        */
     pset_hlhsr_id(PHIGS_HLHSR_ID_ON);     /* Turn on Z buffering           */
     pset_dcue_ind(2); /*allow depth cueing*/
     pset_linewidth(2.0); /*double width*/
     padd_names_set(&nset_names);   /* Add nameset for picking  (PICKABLE)  */
     plabel(LOCAL);                 /* Insert a label for future updates    */
   pset_local_tran3(centermatrix, PTYPE_REPLACE);/*set transformation matrix*/
     plabel(TRANS);                 /* Insert a label for future updates    */
   pset_global_tran3(currmatrix, PTYPE_REPLACE);/*set transformation matrix*/

 if(Lmarkers)
 {
      maximumgroup = maxgrp+1; /* maxgrp+1 does markers */
      if(Lmeasures) maximumgroup = maxgrp+2; /* maxgrp+2 does measures */
      if(Ldrawline) maximumgroup = maxgrp+3; /* maxgrp+3 does drawline */
 }
 else maximumgroup = maxgrp;

 for(ngrp = 1; ngrp <= maximumgroup ; ngrp++)
 {/*loop over groups*/
  if(igrp[ngrp][0] > 0)
  {/*visible group*/
    grpmin = igrp[ngrp][1];
    grpmax = igrp[ngrp][2];
    for(nset = grpmin; nset <= grpmax; nset++)
    {/*loop over subgroups*/
      if(iset[nset][0] > 0) 
      {/*visible subgroup*/
        setmin = iset[nset][1];
        setmax = iset[nset][2];
        for(nlist = setmin; nlist <= setmax ; nlist++)
        {/*loop over lists*/
          if(ilist[nlist][0] > 0)
          {/*visible list*/
            /* Every list defines a phigs structure */
            pexec_struct((Pint)nlist); 
          }/*visible list*/
        }/*loop over lists*/
      }/*visible subgroup*/
    }/*loop over subgroups*/
  }/*visible group*/
 }/*loop over groups*/
 pset_hlhsr_id(PHIGS_HLHSR_ID_OFF);  /* Turn off Z buffering*/
 pclose_struct();

 ppost_struct(WS, DISPLAY_STRUCT, 1.0); /* Post DISPLAY_STRUCT, prio 1 */

 pupd_ws(WS,PUPD_PERFORM);                  /* Update the workstation */
 pset_edit_mode(PEDIT_REPLACE);   /* Set edit mode to replace    */
 predraw_all_structs(WS, PFLAG_ALWAYS); /* Redraw the structure */
}
/*___updatevisibility()______________________________________________________*/


void make_identity(matrix)
    Pmatrix3 matrix;
{
    matrix[0][0] = 1;
    matrix[0][1] = 0;
    matrix[0][2] = 0;
    matrix[0][3] = 0;
    matrix[1][0] = 0;
    matrix[1][1] = 1;
    matrix[1][2] = 0;
    matrix[1][3] = 0;
    matrix[2][0] = 0;
    matrix[2][1] = 0;
    matrix[2][2] = 1;
    matrix[2][3] = 0;
    matrix[3][0] = 0;
    matrix[3][1] = 0;
    matrix[3][2] = 0;
    matrix[3][3] = 1;
}

void make_viewmatrix(matrix)
    Pmatrix3 matrix;
{
    matrix[0][0] = a11;
    matrix[0][1] = a12;
    matrix[0][2] = a13;
    matrix[0][3] = 0;
    matrix[1][0] = a21;
    matrix[1][1] = a22;
    matrix[1][2] = a23;
    matrix[1][3] = 0;
    matrix[2][0] = a31;
    matrix[2][1] = a32;
    matrix[2][2] = a33;
    matrix[2][3] = 0;
    matrix[3][0] = 0;
    matrix[3][1] = 0;
    matrix[3][2] = 0;
    matrix[3][3] = 1;
}

/****Cleanup()***************************************************************/
void Cleanup()                       /* Cleanup routine when done   */
{
 punpost_all_structs(WS);       /* Unpost all structures on ws */
 pdel_all_structs();            /* Delete all structures */
 pclose_ws(WS);                 /* Close workstation     */
 pclose_phigs();                /* Close PHIGS           */
}  
/*___Cleanup()_______________________________________________________________*/

/****makepexcolours()**********************************************************/
void makepexcolours()
{
  Pcolr_rep  rgbColor;
  int     i;
  Pint    *error_ind;

/*???*/
  i = 10+  1; rgbColor.rgb.red = 65535./65535.; rgbColor.rgb.green =     0; rgbColor.rgb.blue =     0;
  pset_colr_rep(WS,i,&rgbColor);
  i = 10+  2; rgbColor.rgb.red =  6000./65535.; rgbColor.rgb.green = 65535./65535.; rgbColor.rgb.blue =     0;
  pset_colr_rep(WS,i,&rgbColor);
  i = 10+  3; rgbColor.rgb.red = 16000./65535.; rgbColor.rgb.green = 16000./65535.; rgbColor.rgb.blue = 65535./65535.;
  pset_colr_rep(WS,i,&rgbColor);
  i = 10+  4; rgbColor.rgb.red =     0; rgbColor.rgb.green = 62000./65535.; rgbColor.rgb.blue = 58000./65535.;
  pset_colr_rep(WS,i,&rgbColor);
  i = 10+  5; rgbColor.rgb.red = 65535./65535.; rgbColor.rgb.green = 63000./65535.; rgbColor.rgb.blue =     0;
  pset_colr_rep(WS,i,&rgbColor);
  i = 10+  6; rgbColor.rgb.red = 65535./65535.; rgbColor.rgb.green =     0; rgbColor.rgb.blue = 60000./65535.;
  pset_colr_rep(WS,i,&rgbColor);
  i = 10+  7; rgbColor.rgb.red = 65535./65535.; rgbColor.rgb.green = 65535./65535.; rgbColor.rgb.blue = 65535./65535.;
  pset_colr_rep(WS,i,&rgbColor);
  i = 10+  8; rgbColor.rgb.red = 65535./65535.; rgbColor.rgb.green = 26000./65535.; rgbColor.rgb.blue = 30000./65535.;
  pset_colr_rep(WS,i,&rgbColor);
  i = 10+  9; rgbColor.rgb.red = 65535./65535.; rgbColor.rgb.green = 24000./65535.; rgbColor.rgb.blue =     0;
  pset_colr_rep(WS,i,&rgbColor);
  i = 10+ 10; rgbColor.rgb.red = 36000./65535.; rgbColor.rgb.green = 14000./65535.; rgbColor.rgb.blue = 65535./65535.;
  pset_colr_rep(WS,i,&rgbColor);
  i = 10+ 11; rgbColor.rgb.red = 15000./65535.; rgbColor.rgb.green = 37000./65535.; rgbColor.rgb.blue = 65535./65535.;
  pset_colr_rep(WS,i,&rgbColor);
  i = 10+ 12; rgbColor.rgb.red = 45000./65535.; rgbColor.rgb.green = 30000./65535.; rgbColor.rgb.blue = 23000./65535.;
  pset_colr_rep(WS,i,&rgbColor);
  i = 10+ 13; rgbColor.rgb.red = 32000./65535.; rgbColor.rgb.green = 32000./65535.; rgbColor.rgb.blue = 32000./65535.;
  pset_colr_rep(WS,i,&rgbColor);
  i = 10+ 14; rgbColor.rgb.red = 35000./65535.; rgbColor.rgb.green = 61000./65535.; rgbColor.rgb.blue =     0;
  pset_colr_rep(WS,i,&rgbColor);
  i = 10+ 15; rgbColor.rgb.red = 65535./65535.; rgbColor.rgb.green = 40000./65535.; rgbColor.rgb.blue =     0;
  pset_colr_rep(WS,i,&rgbColor);
  i = 10+ 16; rgbColor.rgb.red = 63000./65535.; rgbColor.rgb.green = 63000./65535.; rgbColor.rgb.blue = 30000./65535.;
  pset_colr_rep(WS,i,&rgbColor);
  i = 10+ 17; rgbColor.rgb.red =     0; rgbColor.rgb.green = 64000./65535.; rgbColor.rgb.blue = 28000./65535.;
  pset_colr_rep(WS,i,&rgbColor);
  i = 10+ 18; rgbColor.rgb.red = 65535./65535.; rgbColor.rgb.green = 44000./65535.; rgbColor.rgb.blue = 48000./65535.;
  pset_colr_rep(WS,i,&rgbColor);
  i = 10+ 19; rgbColor.rgb.red = 45000./65535.; rgbColor.rgb.green = 55000./65535.; rgbColor.rgb.blue = 65535./65535.;
  pset_colr_rep(WS,i,&rgbColor);
  i = 10+ 20; rgbColor.rgb.red = 39000./65535.; rgbColor.rgb.green = 65535./65535.; rgbColor.rgb.blue = 46000./65535.;
  pset_colr_rep(WS,i,&rgbColor);
  i = 10+ 21; rgbColor.rgb.red = 65535./65535.; rgbColor.rgb.green =     0; rgbColor.rgb.blue = 26000./65535.;
  pset_colr_rep(WS,i,&rgbColor);
}
/*___makepexcolours()_________________________________________________________*/


/****resetalltransformations()************************************************/
void resetalltransformations()
{
  cntr.x =  0.0; /* center point of transformations*/
  cntr.y =  0.0; /* is the origin of any space: 0,0,0 */
  cntr.z =  0.0;
  cntrlocal.x =  0.0;	/* center point of transformations*/
  cntrlocal.y =  0.0;	/* is the origin, 0,0,0 */
  cntrlocal.z =  0.0;
  scalep.delta_x = scalenew/300; /* initial scale is 1 */
  scalep.delta_y = scalenew/300; /*scalenew from MAGEINPT.C*/
  scalep.delta_z = scalenew/300;
  /*scalep.delta_x = scalenew/150;*/ /* initial scale is 1 */
  /*scalep.delta_y = scalenew/150;*/ /*scalenew from MAGEINPT.C*/
  /*scalep.delta_z = scalenew/150;*/
  scalelocal.delta_x = 1.0; /* initial scale is 1 */
  scalelocal.delta_y = 1.0; /*local matrix used only for centering*/
  scalelocal.delta_z = 1.0;
  trans.delta_x = 0.0; /* initial translation by knobs is 0 */
  trans.delta_y = 0.0;
  trans.delta_z = 0.0;
  translocal.delta_x = -fxcenternew;
  translocal.delta_y = -fycenternew;
  translocal.delta_z = -fzcenternew;
  /*rotation matrix is accummulated from changes in the rotation angles*/
  /*has to be reset to current view's starting rotation matrix*/
  /*resetalltransformations() expects resetrot() to have been called*/
  /* so MAGE matrix "a" has been just reloaded */
  /*note: the internal MAGE matrix is LH to compensate for MAC,PC LH coords*/

      rotonlymatrix[0][0] =  a11;
      rotonlymatrix[1][0] = -a12;
      rotonlymatrix[2][0] =  a13;
      rotonlymatrix[3][0] = 0;

      rotonlymatrix[0][1] =  a21;
      rotonlymatrix[1][1] = -a22;
      rotonlymatrix[2][1] =  a23;
      rotonlymatrix[3][1] = 0;

      rotonlymatrix[0][2] =  a31;
      rotonlymatrix[1][2] = -a32;
      rotonlymatrix[2][2] =  a33;
      rotonlymatrix[3][2] = 0;

      rotonlymatrix[0][3] = 0;
      rotonlymatrix[1][3] = 0;
      rotonlymatrix[2][3] = 0;
      rotonlymatrix[3][3] = 1;
}
/*___resetalltransformations()________________________________________________*/

/****resetpex()*************************************************************/
void resetpex()
{
/*
  Called from the reset view call backs:  it re-initializes the
  parameters that define the transformation matrix corresponding to
  translate, rotate and scale operations.  The posted structures are also
  re-initialized since picking elements could have caused some elements to
  be deleted.
*/
      makepexstructures(); /*MUXPHIG.c, reinitialize posted structures*/
      updatevisibility();  /*MUXPHIG.c*/
      /* reset xformation parameters */
      resetalltransformations(); /*MUXPHIG.c*/
      knobmotionhandler(); /*MUXPHIG.c*/
}
/*___resetpex()____________________________________________________________*/

/****setpexmeasures()*******************************************************/
void     setpexmeasures()
{
    int    k;

              /*rebuild PEX structure*/
              /*measures are in maxgrp+2*/
              nset=igrp[maxgrp+2][1];
              /*first do connected lines showing what has been measured*/
              nlist=iset[nset][1]; /*first list in measures subgroup set */
              k = 1 + nxyz + nmarkers;
              pdel_struct((Pint)nlist); /*kill previous list structure*/
              popen_struct((Pint)nlist);
                pset_line_colr_ind(7); /*color*/
                /* number of points in list so far: domeasures() mageangl.c*/
                /*note: fence post problem*/
                pxyz_list[k].num_points =
                        (ilist[maxlist+5][2] - ilist[maxlist+5][1]+1);
                /* Pointer to point array at first point of polyline*/
                pxyz_list[k].points = pxyz+k;
                /* Create a polyline element */
                ppolyline3(&pxyz_list[k]);
              pclose_struct();
              popen_struct(DISPLAY_STRUCT);
                pexec_struct((Pint)nlist);/*structure name is list number*/
              pclose_struct();
              /*second do average points */
              nlist=iset[nset][2]; /*second and last list in measures set */
              k = 5 + nxyz + nmarkers;
              pdel_struct((Pint)nlist); /*kill previous list structure*/
              popen_struct((Pint)nlist);
                pset_line_colr_ind(21); /*color*/
                /* number of points in list so far: domeasures() mageangl.c*/
                pxyz_list[k].num_points =
                        (ilist[maxlist+6][2] - ilist[maxlist+6][1] +1);
                /* Pointer to point array at first point of polymarkers*/
                pxyz_list[k].points = pxyz+k;
                /* Create a polymarker element */
                ppolymarker3(&pxyz_list[k]);
              pclose_struct();
              popen_struct(DISPLAY_STRUCT);
                pexec_struct((Pint)nlist);/*structure name is list number*/
              pclose_struct();
              predraw_all_structs(WS, PFLAG_ALWAYS);/*Redraw the structure*/
	      pupd_ws(WS, PUPD_PERFORM);	/* cause an update */
              updatevisibility(); /*this may not do anything here */
}
/*___setpexmeasures()________________________________________________________*/


/*
  enter_handler

  This event handler restores the knob labels to our labels when the
  pointer moves into the drawing window.  Conditionally labelling the dials
  in this way gives extra feedback to the user that the dials are active
  only when the mouse is inside the appropriate window.
*/
void enter_handler(Widget widget,caddr_t client_data,XEvent *event,Boolean *continue_to_dispatch)
/*     Widget widget;                 */
/*     caddr_t client_data;           */
/*     XEvent *event;                 */
/*     Boolean *continue_to_dispatch; */
{
  register int i;
  XStringFeedbackControl strfc;

  strfc.class = StringFeedbackClass;	/* initialize the feedback structure */
  strfc.length = sizeof(XStringFeedbackControl);
  strfc.num_keysyms = CHARS_PER_DIAL;

  for (i = 0; i < MAXDIALS; i++)	/* change each dial */
    {
      strfc.id = i;			/* id is the knob to change */
      strfc.syms_to_display = knob_labels[i];
      XChangeFeedbackControl(dpy, dials, DvString, &strfc);
    }
}

/*
  leave_handler

  This event handler blanks the knob labels when the pointer leaves the
  drawing window.
*/
void leave_handler(Widget widget,caddr_t client_data,XEvent *event,Boolean *continue_to_dispatch)
{
  static XStringFeedbackControl strfc;
  static KeySym blanks[CHARS_PER_DIAL];
  static Boolean initialized = False;
  register int i;

  if (!initialized)			/* initialize variables the first */
					/* time we're called  */
    {
      for (i = 0; i < CHARS_PER_DIAL; i++)
	blanks[i] = (KeySym) ' ';	/* prepare a blank keysym array */

      strfc.class = StringFeedbackClass; /* initialize the feedback struct */
      strfc.length = sizeof(XStringFeedbackControl);
      strfc.num_keysyms = CHARS_PER_DIAL;
      strfc.syms_to_display = blanks;

      initialized = True;		/* remember we've been initialized */
    }

  for (i = 0; i < MAXDIALS; i++)	/* for each dial */
    {
      strfc.id = i;			/* indicate which dial to change */
      XChangeFeedbackControl(dpy, dials, DvString, &strfc); /* change it */
    }
}


/*
  open_knob

  This routine opens the knob box on the given display and selects device
  motion extension events on the given window.  Extension events are
  selected by first invoking the appropriate macro on an XEventClass
  structure (in this case DeviceMotionNotify) and then calling
  XSelectExtensionEvent.
*/
void open_knob()
{
  /* dial labels as ascii strings, in CHAOS order */
  static char textstring[MAXDIALS][CHARS_PER_DIAL] = {
    " X TRAN ", " Y TRAN ", " Z TRAN ",	"ZOOM1.00",
    " Y ROT  ", " X ROT  ", " Z ROT  ", "SLAB 200" 
    };
  register int knob, i;
  int ndevices;				/* number of extension devices */
  XDeviceInfo *devices = NULL;		/* extension device info list */
  Atom dials_atom = XInternAtom(dpy, "KNOB_BOX", True);
					/* intern device name into an atom */
  XID dials_id;				/* device ID for dials box */
  XEventClass eventClass[1];

  dials = NULL; /*so can tell if manage to open this XDevice */

  devices = XListInputDevices(dpy, &ndevices); /* get list of devices */
  
  for (i = 0; i < ndevices; i++, devices++)
    if ((devices->type == dials_atom) && (devices->use == IsXExtensionDevice))
      {					/* did we find the dial box? */
	dials_id = devices->id;		/* yes, remember its device ID */
	dials = XOpenDevice(dpy, dials_id); /* and open it */
	break;				/* we only want the first one... */
      }
  
  if (!dials)				/* couldn't open or find dials */
    {
      printf("?couldn't open dials box.\n");
      exit(1);
    }
					/* select device motion events */
  DeviceMotionNotify(dials, DeviceMotion, eventClass[0]);
  XSelectExtensionEvent(dpy, drawWindow, eventClass, 1);
  XSelectExtensionEvent(dpy, XtWindow(grafwindow), eventClass, 1);

  for (knob = 0; knob < MAXDIALS; knob++) /* convert ascii labels to KeySyms */
    for (i = 0; i < CHARS_PER_DIAL; i++)
      knob_labels[knob][i] = (KeySym) textstring[knob][i];
}

/*
  init_pick

  This routine initializes the picking extension.  It must be called after
  the pick context is established (in routine SetupWorkstation, file phigs4.c)
  and before event processing is begun.

  Major_op is the major opcode of the picking extension.  First_ev is the
  first event number dynamically allocated for the extension.  First_err is
  the first error number dynamically allocated for the extension.
*/
void init_pick()
{					/* query picking extension */
					/* for major opcode, first event */
  XQueryExtension(dpy, PICKNAME, &Major_op, &First_ev, &First_err);

					/* Set the picking mode to return */
					/* the item picked that is nearest */
  XSetPickMode(dpy, pc, pick_near);	/* in Z */
  
  XSetPickBoxSize(dpy, pc, 5, 5);	/* Set pick box size to 5x5 pixels */
					/* Cause all of picked item to be */
					/* highlighted */
  XSetPickHighlightingMode(dpy, pc, pick_highlighting_command);
					/* Select pick and pick path events */
					/* to be returned */
  XSelectPickEvents(dpy, drawWindow, PickMask);
}


/****knobmotionhandler()*******************************************************/
void knobmotionhandler()
{/*knob motion handler*/
    int   i,j;
    float scalingnumber,zclippingnumber;
  XStringFeedbackControl strfc;

  strfc.class = StringFeedbackClass;	/* initialize the feedback structure */
  strfc.length = sizeof(XStringFeedbackControl);
  strfc.num_keysyms = CHARS_PER_DIAL;


          /* Process device motion events, knobs arrangement: CHAOS */
          /* Dial 0 input check trans x */
          trans.delta_x += (knob_totals[0]-knob_hold[0]) * DIALSCALE;
          
          /* Dial 1 input check trans y */
          trans.delta_y += (knob_totals[1]-knob_hold[1]) * DIALSCALE;
          
	  /* Dial 2 input check trans z */
	  trans.delta_z += (knob_totals[2]-knob_hold[2]) * DIALSCALE;
          /*vector trans accummulates the translations */
          
	  /* Dial 3 input check uniform scale */
	  {
              /*scalingnumber = (knob_totals[3]-knob_hold[3]) *0.1*DIALSCALE;*/
              scalingnumber=(knob_totals[3]-knob_hold[3])*DIALSCALE*0.01*scale;
	      scalep.delta_x += scalingnumber;
	      scalep.delta_y += scalingnumber;
	      scalep.delta_z += scalingnumber;
              if(scalep.delta_x < 0.0)
              {scalep.delta_x=0.0;scalep.delta_y=0.0;scalep.delta_z=0.0;}
              sprintf(word,"ZOOM%4.2f",(float)(scalep.delta_x*150.0/scale));
              for(j=0;j<8;j++) knob_labels[3][j] = word[j];
              strfc.id = 3; /* id is the knob to change */
              strfc.syms_to_display = knob_labels[3];
              XChangeFeedbackControl(dpy, dials, DvString, &strfc);
	  }
	  /* Dial 4 input check rot y */
          roty=(knob_hold[4]-knob_totals[4])*50.0*DIALSCALE;
               /*just do incremental changes*/
               /*matrix is accummulated, not the angles*/
	  /* dial 5 input check rot x */
	  rotx = (knob_hold[5]-knob_totals[5])*50.0*DIALSCALE;
          
	  /* Dial 6 input check rot z */
	  rotz = (knob_hold[6]-knob_totals[6])*50.0*DIALSCALE;

/*???*/
	  /* Dial 7 input zclipping planes */
	  {
	    if(knob_totals[7]-knob_hold[7]) /*change?*/
            {
	      izclip += (knob_totals[7]-knob_hold[7]); /*integers*/
              if(izclip > 200) izclip = 200;
              if(izclip < 0  ) izclip = 0;
              zclippingnumber = (float)izclip/400.0;
              /*vrep limits must be 0.0 <= . <= 1.0 */
              vrep.clip_limit.z_min = 0.5 - zclippingnumber;
              vrep.clip_limit.z_max = 0.5 + zclippingnumber;
              pset_view_rep3( WS, VIEW, &vrep );/*Set the view representation*/
              sprintf(word,"SLAB%4d",izclip);
              for(j=0;j<8;j++) knob_labels[7][j] = word[j];
              strfc.id = 7; /* id is the knob to change */
              strfc.syms_to_display = knob_labels[7];
              XChangeFeedbackControl(dpy, dials, DvString, &strfc);
            }
	  }

          for(i=0;i<MAXDIALS;i++) 
          {/*reset comparison knob postions*/
              knob_hold[i]=knob_totals[i];
          }

          /*build new individual rotation matrices for each axis */
          protate_x(DEG_TO_RAD(rotx),&PEX_error,mx);
          protate_y(DEG_TO_RAD(roty),&PEX_error,my);
          protate_z(DEG_TO_RAD(rotz),&PEX_error,mz);
          /*Build the transformation matrix*/
          /*rotonlymatrix accummulates the 3x3 rotation part in a 4x4 matrix*/
          /*order of matrix multiplication does really matter */
          pcompose_matrix3(mx,rotonlymatrix,&PEX_error,tempmatrix);
          pcompose_matrix3(my,tempmatrix,&PEX_error,holdmatrix);
          pcompose_matrix3(mz,holdmatrix,&PEX_error,rotonlymatrix);
          /*now rotonlymatrix has been updated for these angle changes*/
          
          /*set up a temporary matrix to hold the scale and transl part */
          pbuild_tran_matrix3(&cntr, &trans, 0.0, 0.0 ,0.0
             , &scalep, &PEX_error ,tempmatrix);
          /*tempmatrix now has new edges and diagonal for scalep and transl*/
          /*set up a pure translation matrix to do centering*/
          translocal.delta_x = -fxcenternew;
          translocal.delta_y = -fycenternew;
          translocal.delta_z = -fzcenternew;
          pbuild_tran_matrix3(&cntrlocal,&translocal, 0.0, 0.0 ,0.0
             , &scalelocal, &PEX_error ,centermatrix);
          
          /* from the accummulated translations held in "trans" */
          SAFE_PEX("pbuild_tran_matrix3");     /* Check for error status*/
           
          /*order of matrix multiplication keeps transl in screen space */
          pcompose_matrix3(tempmatrix,rotonlymatrix,&PEX_error,currmatrix);
          /*currmatrix now has current working 4x4 stuff*/
          /* Combine old and new matrices*/
          popen_struct(DISPLAY_STRUCT);     /* Open structure for editing*/
	  {
             pset_elem_ptr(0);               /* Reset element pointer    */
             pset_elem_ptr_label(LOCAL);     /* Find transformation label*/
             poffset_elem_ptr(1);            /* Point at matrix          */
             /*local matrix used to pre-center the molecule */
             pset_local_tran3(centermatrix,PTYPE_REPLACE);/*replace*/
             pset_elem_ptr_label(TRANS);     /* Find transformation label*/
             poffset_elem_ptr(1);            /* Point at matrix          */
             /*global matrix used for current knobs rot and transl */
             pset_global_tran3(currmatrix,PTYPE_REPLACE);/*replace matrix*/
	  }
          pclose_struct();                    /* Close structure         */
          predraw_all_structs(WS, PFLAG_ALWAYS); /* Redraw the structure */

}/*knob motion handler*/
/*___knobmotionhandler()______________________________________________________*/

/****pickeventhandler()********************************************************/
void pickeventhandler(int pickstructureid,float pickmodelx,float pickmodely,float pickmodelz)
{/*pick event handler*/
  int i,k,nstartit,nendit,j,jmin,jmax,jpoint;
  double dnsq,dmsq,dist;
      /*find closest point to picked place*/
      nlist = pickstructureid;
      if(nlist>0)
      {/*nlist picked*/
              jmin=ilist[nlist][1];
              jmax=ilist[nlist][2];
              for(j=jmin;j<=jmax;j++)
              {/*check points in this list*/
                  dnsq= (pickmodelx-fxyz(0,j))*(pickmodelx-fxyz(0,j))
                       +(pickmodely-fxyz(1,j))*(pickmodely-fxyz(1,j))
                       +(pickmodelz-fxyz(2,j))*(pickmodelz-fxyz(2,j));
                  if(j>jmin)
                  {
                      if(dnsq < dmsq){dmsq=dnsq;jpoint=j;}
                  }
                  else               {dmsq=dnsq;jpoint=j;}
              }/*check points in this list*/
          pickvec = jpoint;
          /*get pointID*/
          /*copy char str from storage */
          for(k=0; k<(MAXPTIDCHAR+1); k++)
          {
              icom = (long)(MAXPTIDCHAR+1)*(long)pickvec + (long)k;
                /*note: define  pcom1(icom)  *(com + icom)  */
              comstr[k] = ( pcom1(icom) );
              grafselection[0][k]=grafselection[1][k]; /* shift to previous*/
              grafselection[1][k]=comstr[k]; /* load this pick point ID*/
                 /*grafselection is for hypertext question answers */
          }
          comstr[(MAXPTIDCHAR)] = '\0'; /*should already be this anyway*/
          /* use original coord for distance calculation*/
          dist= sqrt(
                   (oldxyz[1]-fxyz(0,pickvec))*(oldxyz[1]-fxyz(0,pickvec)) +
                   (oldxyz[2]-fxyz(1,pickvec))*(oldxyz[2]-fxyz(1,pickvec)) +
                   (oldxyz[3]-fxyz(2,pickvec))*(oldxyz[3]-fxyz(2,pickvec))  );
          distmeasure = dist; /*constructline needs global distmeasure*/
          if(Lmeasureson) /*Lmeasures only if markers possible as well*/
          {
              domeasures();  /*MAGEANGL.C*/
              /*puts anglemeasure,dihedralmeasure into common storage*/
/*c*/         sprintf(dstring,", dist=%.3lf, ang=%.1f, dhr=%.1f\0",
              dist,anglemeasure,dihedralmeasure);
              /*replace the measures items in the PEX accessible list*/
              for(k=1+nxyz+nmarkers; k<=maxmeasures+nxyz+nmarkers; k++)
              {
                  pxyz[k].x = fxyz(0,k);
                  pxyz[k].y = fxyz(1,k);
                  pxyz[k].z = fxyz(2,k);
              }
              /*rebuild PEX structure*/
              /*measures are in maxgrp+2*/
              nset=igrp[maxgrp+2][1];
              /*first do connected lines showing what has been measured*/
              nlist=iset[nset][1]; /*first list in measures subgroup set */
              k = 1 + nxyz + nmarkers;
              pdel_struct((Pint)nlist); /*kill previous list structure*/
              popen_struct((Pint)nlist);
                pset_line_colr_ind(7); /*color*/
                /* number of points in list so far: domeasures() mageangl.c*/
                /*note: fence post problem*/
                pxyz_list[k].num_points =
                        (ilist[maxlist+5][2] - ilist[maxlist+5][1]+1);
                /* Pointer to point array at first point of polyline*/
                pxyz_list[k].points = pxyz+k;
                /* Create a polyline element */
                ppolyline3(&pxyz_list[k]);
              pclose_struct();
              popen_struct(DISPLAY_STRUCT);
                pexec_struct((Pint)nlist);/*structure name is list number*/
              pclose_struct();
              /*second do average points */
              nlist=iset[nset][2]; /*second and last list in measures set */
              k = 5 + nxyz + nmarkers;
              pdel_struct((Pint)nlist); /*kill previous list structure*/
              popen_struct((Pint)nlist);
                pset_line_colr_ind(21); /*color*/
                /* number of points in list so far: domeasures() mageangl.c*/
                pxyz_list[k].num_points =
                        (ilist[maxlist+6][2] - ilist[maxlist+6][1] +1);
                /* Pointer to point array at first point of polymarkers*/
                pxyz_list[k].points = pxyz+k;
                /* Create a polymarker element */
                ppolymarker3(&pxyz_list[k]);
              pclose_struct();
              popen_struct(DISPLAY_STRUCT);
                pexec_struct((Pint)nlist);/*structure name is list number*/
              pclose_struct();
              predraw_all_structs(WS, PFLAG_ALWAYS);/*Redraw the structure*/
	      pupd_ws(WS, PUPD_PERFORM);	/* cause an update */
              updatevisibility(); /*this may not do anything here */
          }
          else sprintf(dstring,", dist=%.3lf\0",dist);
          oldxyz[1] = fxyz(0,pickvec); /* use original coord */
          oldxyz[2] = fxyz(1,pickvec);
          oldxyz[3] = fxyz(2,pickvec);
          /*send comstr==pointID and dist==distance to message place*/
printf("%s  %s",comstr,dstring);
          sprintf(word,"%s  %s",comstr,dstring);
          XmTextSetString(message,word);

          if(Lpickon)
          {
                  fxcenternew = fxyz(0,pickvec);
                  fycenternew = fxyz(1,pickvec);
                  fzcenternew = fxyz(2,pickvec);
          }
          if(Lmarkers)
          {/*marker placing*/
              if(imarker == 1){nstartit=1;nendit= 6;}
              else            {nstartit=7;nendit=12;}
              for(n=nstartit; n<=nendit ; n++)
              {/*place marker*/
                  k = n + nxyz;
                  xyz(0,k) = marker[n][0];
                  /*recreate pseudo-original float markers */
                  /*at new position*/
                  fxyz(0,k) = fmarker[n][0]  + fxyz(0,pickvec);
                  fxyz(1,k) = fmarker[n][1]  + fxyz(1,pickvec);
                  fxyz(2,k) = fmarker[n][2]  + fxyz(2,pickvec);
                  xyz(1,k) = scalenew*(fxyz(0,k)-fxcenternew);
                  xyz(2,k) = scalenew*(fxyz(1,k)-fycenternew);
                  xyz(3,k) = scalenew*(fxyz(2,k)-fzcenternew);
                  pxyz[k].x = fxyz(0,k);
                  pxyz[k].y = fxyz(1,k);
                  pxyz[k].z = fxyz(2,k);
              }/*place marker*/
              /*rebuild PEX structure*/
              k = nstartit + nxyz;
              /*markers are in maxgrp+1*/
              nset=igrp[maxgrp+1][1];
              nlist=iset[nset][1];
              if(imarker==2) nlist = nlist+1; /*second list*/
              pdel_struct((Pint)nlist); /*kill previous marker*/
              popen_struct((Pint)nlist);
                pset_line_colr_ind(3); /*color*/
                /* Fill in number of points in list */
                pxyz_list[k].num_points = 6;
                /* Pointer to point array at first point of polyline*/
                pxyz_list[k].points = pxyz+k;
                /* Create a polyline element */
                ppolyline3(&pxyz_list[k]);
              pclose_struct();
              popen_struct(DISPLAY_STRUCT);
                pexec_struct((Pint)nlist);/*structure name is list number*/
              pclose_struct();
              predraw_all_structs(WS, PFLAG_ALWAYS);/*Redraw the structure*/
	      pupd_ws(WS, PUPD_PERFORM);	/* cause an update */
              updatevisibility(); /*this may not do anything here */
              /*set flags for next marker placement*/
              if(imarker == 1)imarker=2;
              else            imarker=1;
          }/*marker placing*/
          /*pretend a knob motion has occurred so transformations will update*/
          knobmotionhandler();
      }/*nlist picked*/

}/*pick event handler*/
/*___pickeventhandler()_______________________________________________________*/

/****OpenPex()****************************************************************/
void OpenPex(Display *dpy)
{
  Pxphigs_info  xinfo;
  unsigned long  infomask;

  xinfo.display = dpy;
  xinfo.rmdb    = NULL;
  xinfo.appl_id.name = NULL;
  xinfo.appl_id.class = NULL;
  xinfo.args.argc_p    = NULL;
  xinfo.args.argv      = NULL;
  xinfo.flags.no_monitor = 1;
  xinfo.flags.force_client_SS = 0;

  infomask = PXPHIGS_INFO_DISPLAY | PXPHIGS_INFO_FLAGS_NO_MON;
  popen_xphigs((char*)NULL, PDEF_MEM_SIZE, infomask, &xinfo);
}
/*___OpenPex()_______________________________________________________________*/
 
/****OpenPhigs()**************************************************************/
void  OpenPhigs()   /*MUXPHIG.c*/
{
  int  i;

  open_knob();   /*MUXPHIG.c*/

  OpenPex(dpy);  /*MUXPHIG.c, old StartPhigs from mesvphig.c*/
  SetupWorkstation(dpy,drawWindow);/*MUXPHIG.c, old StartPhigs from mesvphig.c*/
  makepexcolours(); /*MUXPHIG.C, old StartPhigs from mesvphig.c*/ 
                    /*then:  old StartPhigs from mesvphig.c */
                    /*called Get_events(dpy) in mesvmotf.c  (master-loop)*/
                    /*and when that ended, called Cleanup() in mesvphig.c */
  for(i=0; i<MAXDIALS; i++) /* Initialize knob values array (from Get_events*/
  {
      knob_totals[i] = 0;
      knob_hold[i] = 0;
  }
  resetalltransformations(); /*MUXPHIG.c,     from Getevents before loop*/
  pset_edit_mode(PEDIT_REPLACE); /*Set edit mode to replace elements, " */
  init_pick();   /*MUXPHIG.c*/   /* initialize picking              , " */
}
/*___OpenPhigs()_____________________________________________________________*/

/****dospecialevent()********************************************************/
void  dospecialevent(XEvent *areport)  /*MUXPHIG.c*/
{
      if (areport->type == DeviceMotion) /* handle device motion event */
          domotionevent(areport);  /*MUXPHIG.c*/
      else if (areport->type == First_ev + XPickEventOffset)/*picking event*/
          dopickingevent(areport); /*MUXPHIG.c*/
      else
          ;
}
/*___dospecialevent()________________________________________________________*/

/****domotionevent()*********************************************************/
void    domotionevent(XEvent *areport)  /*MUXPHIG.c*/
{/*motion event*/
    int  i;

	  XDeviceMotionEvent *dm = (XDeviceMotionEvent *) areport;
	  do				/* Compress motion events */
	    for (i = 0; i < dm->axes_count; i++)
	      knob_totals [dm->first_axis+i] += dm->axis_data[i];
	  while (XCheckTypedEvent(dpy, DeviceMotion, dm));
          knobmotionhandler(); /*MUXPHIG.c*/

}/*motion event*/
/*___domotionevent()_________________________________________________________*/

/****dopickingevent()*********************************************************/
void    dopickingevent(XEvent *areport) /*MUXPHIG.c*/
{/*picking event*/
	  XPickEvent *pick = (XPickEvent *) areport;

	  popen_struct(pick->structureid); /* open picked structure */
	  {
	    pset_elem_ptr(0);		/* reset the element points */
            poffset_elem_ptr(pick->elementid);/*set the element pointer*/
					/* to the picked element  */
	    /*pdel_elem();*/		/* delete the picked element */
	  }
	  pclose_struct();
          pickeventhandler(pick->structureid
                 ,pick->modelx,pick->modely,pick->modelz);
}/*picking event*/
/*___dopickingevent()________________________________________________________*/

/****doxpick()*********************************************************/
void    doxpick(Widget thewindow, int ix, int iy) /*MUXPHIG.c*/
{
    XPick(dpy, thewindow, pc, ix, iy);
}
/*___doxpick()________________________________________________________*/

/****doxprepick()*********************************************************/
void    doxprepick(Widget thewindow, int ix, int iy) /*MUXPHIG.c*/
{
    XPrePick(dpy, thewindow, pc, ix, iy);
}
/*___doxprepick()________________________________________________________*/

/****doxpickpredraw()*********************************************************/
void    doxpickpredraw() /*MUXPHIG.c*/
{
    predraw_all_structs(WS, PFLAG_ALWAYS);
}
/*___doxpickpredraw()________________________________________________________*/

/****setpexclipplanes()*******************************************************/
void  setpexclipplanes()  /*MUXPHIG.c*/
{
    if(Lzclipon)
        vrep.back_clip = vrep.front_clip = PIND_CLIP;
    else
        vrep.back_clip = vrep.front_clip = PIND_NO_CLIP;
    pset_view_rep3( WS, VIEW, &vrep );/*Set the view representation*/
} 
/*___setpexclipplanes()______________________________________________________*/
